//AmpPhaseDataset.java:  JFreeChart dataset of amplitude or phase data
//                       points generated by JEvalResp.
//
//  5/14/2002 -- [ET]  Initial version.
//   7/5/2002 -- [KF]  Remove selected index from the data set.
//   5/8/2002 -- [ET]  Changed "com.jrefinery" to "org.jfree" for
//                     JFreeChart 0.9.8.
//  3/30/2005 -- [ET]  Added ability to represent multiple responses (via
//                     "RespArraysInfo []" parameter to constructor);
//                     modified to return only amplitude or only phase
//                     values (decided via 'amplitudeSeriesFlag'); modified
//                     to work with JFreeChart v1.0; added tooltip
//                     generator.
//  4/30/2007 -- [ET]  Renamed 'getSeriesName()' method to 'getSeriesKey()'.
//

package com.isti.jplotresp;

import java.util.ArrayList;
import org.jfree.data.DomainOrder;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.general.AbstractSeriesDataset;
import org.jfree.chart.labels.XYToolTipGenerator;
import com.isti.jevalresp.AmpPhaseBlk;
import com.isti.jevalresp.RespArraysInfo;
import java.text.DecimalFormat;
import java.text.NumberFormat;

/**
 * Class AmpPhaseDataset is a JFreeChart dataset of amplitude or phase
 * data points generated by JEvalResp.  A single response or multiple
 * responses may be represented, with amplitude or phase for each
 * response.
 */
public class AmpPhaseDataset extends AbstractSeriesDataset
                                                        implements XYDataset
{
  protected final RespArraysInfo [] respArraysInfoArray;
  protected final double [] frequencyArray;
  protected final boolean amplitudeSeriesFlag;
  protected final int seriesCountValue;
  protected final AmpPhaseTooltipGenerator ampPhaseTooltipGeneratorObj;

         //setup number formatter objects use by tooltip generator:
  private static final NumberFormat freqNumberFormatObj =
                                                 NumberFormat.getInstance();
  private static final NumberFormat ampNumberFormatObj =
                                                 NumberFormat.getInstance();
  private static final NumberFormat phaseNumberFormatObj =
                                                 NumberFormat.getInstance();
  static      //static initialization block; executes only once
  {                     //set patterns for number formatter objects:
    if(freqNumberFormatObj instanceof DecimalFormat)
      ((DecimalFormat)freqNumberFormatObj).applyPattern("0.000E0");
    if(ampNumberFormatObj instanceof DecimalFormat)
      ((DecimalFormat)ampNumberFormatObj).applyPattern("0.000000E00");
    if(phaseNumberFormatObj instanceof DecimalFormat)
      ((DecimalFormat)phaseNumberFormatObj).applyPattern("0.000");
  }

  /**
   * Constructs a dataset of amplitude or phase values for a response,
   * including its individual stages.
   * @param rArrsInfoArr array of response-arrays/information objects.
   * @param ampSeriesFlag true for amplitude series; false for phase series.
   */
  public AmpPhaseDataset(RespArraysInfo [] rArrsInfoArr,
                                                      boolean ampSeriesFlag)
  {
    if(rArrsInfoArr == null)                     //if null pointer then
      throw new NullPointerException();          //throw exception
    amplitudeSeriesFlag = ampSeriesFlag;
    if(rArrsInfoArr.length > 0)
    {    //array contains entries
              //check if any array entries are null:
      boolean nullEntryFlag = false;
      for(int i=0; i<rArrsInfoArr.length; ++i)
      {  //for each entry in given array
        if(rArrsInfoArr[i] == null)
        {
          nullEntryFlag = true;
          break;
        }
      }
      if(nullEntryFlag)
      {  //at least one null array entry
              //remove null array entries:
        final ArrayList listObj = new ArrayList();
        for(int i=0; i<rArrsInfoArr.length; ++i)
        {     //for each entry in given array
          if(rArrsInfoArr[i] != null)            //if entry not null then
            listObj.add(rArrsInfoArr[i]);        //add to list
        }
        final RespArraysInfo [] rAInfArr =       //create new array
                                         new RespArraysInfo[listObj.size()];
              //convert list to new array (without null entries):
        respArraysInfoArray = (RespArraysInfo [])(listObj.toArray(rAInfArr));
      }
      else    //no null entries; save handle to given array
        respArraysInfoArray = rArrsInfoArr;
      if(respArraysInfoArray.length > 0)
      {  //array is not empty; fetch frequency array from first entry
        frequencyArray = respArraysInfoArray[0].frequencyArr;
                                       //setup series count:
        seriesCountValue = respArraysInfoArray.length;
        if(frequencyArray == null)     //if null ptr then throw exception
          throw new NullPointerException("No frequency data");
                   //create tooltip generator object:
        ampPhaseTooltipGeneratorObj = new AmpPhaseTooltipGenerator();
        return;
      }
    }
    throw new IllegalArgumentException("No response data");
  }

  /**
   * Constructs a dataset of amplitude and phase values for a response.
   * @param freqArr array of frequency values used to calculate the
   * data.
   * @param ampPhaseBlkArr array of JEvalResp 'AmpPhaseBlk' objects.
   * @param ampSeriesFlag true for amplitude series; false for phase series.
   */
  public AmpPhaseDataset(double [] freqArr, AmpPhaseBlk [] ampPhaseBlkArr,
                                                      boolean ampSeriesFlag)
  {
    if(freqArr == null || ampPhaseBlkArr == null)     //if null pointer then
      throw new NullPointerException();               //throw exception
    frequencyArray = freqArr;                         //save freq array
    amplitudeSeriesFlag = ampSeriesFlag;
         //create array with single entry for response:
    respArraysInfoArray = new RespArraysInfo[]
                { new RespArraysInfo(0,freqArr,"Response",ampPhaseBlkArr) };
    seriesCountValue = 1;    //indicate 1 series
                   //create tooltip generator object:
    ampPhaseTooltipGeneratorObj = new AmpPhaseTooltipGenerator();
  }

  /**
   * Returns the specified frequency value.
   * @param series index of the required series (not used).
   * @param item index (zero-based) of the required item.
   * @return A new 'Number' object holding the value.
   */
  public double getXValue(int series, int item)
  {
    return frequencyArray[item];
  }

  /**
   * Returns the specified frequency value.
   * @param series index of the required series.
   * @param item index (zero-based) of the required item.
   * @return A new 'Number' object holding the value.
   */
  public Number getX(int series, int item)
  {
    return new Double(getXValue(series,item));
  }

  /**
   * Returns the specified amplitude or phase value.
   * @param series index of the required series.
   * @param item index (zero-based) of the required item.
   * @return A new 'Number' object holding the value.
   */
  public double getYValue(int series, int item)
  {
              //select array entry with response-amp/phase values:
    final RespArraysInfo respArrsInfoObj = respArraysInfoArray[series];
              //select and return amp/phase value:
    return amplitudeSeriesFlag ? respArrsInfoObj.ampPhaseArray[item].amp :
                                  respArrsInfoObj.ampPhaseArray[item].phase;
  }

  /**
   * Returns the specified amplitude or phase value.
   * @param series index of the required series.
   * @param item index (zero-based) of the required item.
   * @return A new 'Number' object holding the value.
   */
  public Number getY(int series, int item)
  {
    return new Double(getYValue(series,item));
  }

  /**
   * Returns the number of series in the data source.
   * @return The number of series in the data source.
   */
  public int getSeriesCount()
  {
    return seriesCountValue;
  }

  /**
   * Returns the name of the series.
   * @param series index (zero-based) of the series.
   * @return The name of the series.
   */
  public Comparable getSeriesKey(int series)
  {
         //if more then one set of amp/phase series then
         // fetch response identification string:
    final String nameStr = (seriesCountValue > 1) ?
                       (respArraysInfoArray[series].identifyStr + " ") : "";
              //return ID string + amplitude or phase tag:
    return amplitudeSeriesFlag ? (nameStr +
                     ((series<=0)?"Amplitude":"Amp")) : (nameStr + "Phase");
  }

  /**
   * Returns the number of items in the specified series.
   * @param series index (zero-based) of the series (not used).
   * @return The number of items in the specified series.
   */
  public int getItemCount(int series)
  {
    return frequencyArray.length;
  }

  /**
   * Returns the order of the domain (or X) values returned by the dataset.
   * @return The domain order (DomainOrder.ASCENDING).
   */
  public DomainOrder getDomainOrder()
  {
    return DomainOrder.ASCENDING;
  }

  /**
   * Returns the tooltip generator object for this dataset.
   * @return The 'AmpPhaseTooltipGenerator' object for this dataset.
   */
  public AmpPhaseTooltipGenerator getAmpPhaseTooltipGeneratorObj()
  {
    return ampPhaseTooltipGeneratorObj;
  }

  /**
   * Returns a tooltip string for the given series and item.
   * @param series index of the required series.
   * @param item index (zero-based) of the required item.
   * @return A new tooltip string.
   */
  public String getTooltipStr(int series, int item)
  {
              //select array entry with response-amp/phase values:
    final RespArraysInfo respArrsInfoObj = respArraysInfoArray[series];

    return "f=" + freqNumberFormatObj.format(frequencyArray[item]) + "  " +
                             (amplitudeSeriesFlag ? "Amp" : "Phase") + "=" +
                                                      (amplitudeSeriesFlag ?
                                                  ampNumberFormatObj.format(
                                  respArrsInfoObj.ampPhaseArray[item].amp) :
                                                phaseNumberFormatObj.format(
                               respArrsInfoObj.ampPhaseArray[item].phase)) +
       ((seriesCountValue > 1) ? ("  " + respArrsInfoObj.identifyStr) : "");
  }


  /**
   * Class AmpPhaseTooltipGenerator defines a tooltip generator for
   * the amplitude/phase dataset.
   */
  public class AmpPhaseTooltipGenerator implements XYToolTipGenerator
  {
    /**
     * Generates the tooltip text for the specified item.
     * @param dataset the dataset.
     * @param series series index (zero-based).
     * @param item item index (zero-based).
     * @return The tooltip text (possibly <code>null</code>).
     */
    public String generateToolTip(XYDataset dataset, int series, int item)
    {
      return getTooltipStr(series,item);
    }
  }
}
