//CallbackProcPlot.java:  Implements a callback method that processes and
//                        writes the output of a response.
//
//   5/14/2002 -- [ET]  Initial version.
//   6/27/2002 -- [KF]  Always use linear vertical axis for "phase" plots.
//   7/10/2002 -- [ET]  Changed 'apOutputFlag' to 'respTypeIndex'.
//    8/6/2002 -- [ET]  Changed parameters on 'generateOutput()' to match
//                      new version of 'CallbackProcWrite'; removed
//                      local version of 'responseInfo()' (no longer
//                      needed because 'channelIdObj' now passed into
//                      'generateOutput()' method); modified to pass
//                      'logSpacingFlag' parameter to parent constructor.
//   4/11/2005 -- [ET]  Added try/catch to 'generateOutput()' method;
//                      modified to check for non-positive amplitude
//                      value(s) on logarithmic plot (and switch to
//                      linear plot if found); added 'useDelayFlag' and
//                      'showInputFlag' parameters to constructors;
//                      modified 'generateOutput()' method to always
//                      operate via event-dispatch thread; modified to
//                      create copy of 'plotDisplayOptionPanel' object.
//   11/1/2005 -- [ET]  Added optional List-blockette interpolation
//                      parameters to constructors; modified to always
//                      use 'respTypeIndex' value of 'RESP_AP_TYPEIDX'.
//   5/27/2010 -- [ET]  Added optional parameters 'unwrapPhaseFlag' and
//                      'totalSensitFlag' to constructors.
//  10/29/2013 -- [ET]  Added optional parameter 'b62XValue' to constructors.
//

package com.isti.jplotresp;

import java.util.Date;
import java.io.PrintStream;
import javax.swing.*;
import edu.iris.Fissures.IfNetwork.ChannelId;
import com.isti.util.UtilFns;
import com.isti.jevalresp.RespProcessor;
import com.isti.jevalresp.OutputGenerator;
import com.isti.jevalresp.CallbackProcWrite;

/**
 * Class CallbackProcPlot implements a callback method that processes and
 * writes the output of a response.
 */
public class CallbackProcPlot extends CallbackProcWrite
{
  protected final JTabbedPane tabbedPaneObj;
  protected final PlotDisplayOptionPanel globalPlotDisplayOptionPanelObj;
  protected boolean generateCompletedFlag = false;
  protected boolean generateResultFlag = false;

  /**
   * Constructs a response processing callback object.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param respProcObj the "RespProcessor' object being used.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param listInterpOutFlag true to interpolate amp/phase output
   * from responses containing List blockettes.
   * @param listInterpInFlag true to interpolate amp/phase input from
   * List blockettes in responses (before output is calculated).
   * @param listInterpTension tension value for List-blockette
   * interpolation algorithm.
   * @param unwrapPhaseFlag true to unwrap phase output values.
   * @param totalSensitFlag true to use stage 0 (total) sensitivity;
   * false to use computed sensitivity.
   * @param b62XValue sample value for polynomial blockette (62).
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,
            RespProcessor respProcObj,int outUnitsConvIdx,double [] freqArr,
               boolean logSpacingFlag,boolean verboseFlag,int startStageNum,
                int stopStageNum,boolean useDelayFlag,boolean showInputFlag,
                        boolean listInterpOutFlag, boolean listInterpInFlag,
                           double listInterpTension,boolean unwrapPhaseFlag,
                 boolean totalSensitFlag,double b62XValue,int respTypeIndex,
                                       boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    super(respProcObj,outUnitsConvIdx,freqArr,logSpacingFlag,verboseFlag,
                      startStageNum,stopStageNum,useDelayFlag,showInputFlag,
                       listInterpOutFlag,listInterpInFlag,listInterpTension,
                                  unwrapPhaseFlag,totalSensitFlag,b62XValue,
                                  PlotRun.RESP_AP_TYPEIDX,stdioFlag,outStm);
                                  //11/1/2005:  Always use 'RESP_AP_TYPEIDX'
    this.tabbedPaneObj = tabbedPaneObj;
    this.globalPlotDisplayOptionPanelObj = plotDisplayOptionPanelObj;
  }

  /**
   * Constructs a response processing callback object.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param respProcObj the "RespProcessor' object being used.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param listInterpOutFlag true to interpolate amp/phase output
   * from responses containing List blockettes.
   * @param listInterpInFlag true to interpolate amp/phase input from
   * List blockettes in responses (before output is calculated).
   * @param listInterpTension tension value for List-blockette
   * interpolation algorithm.
   * @param unwrapPhaseFlag true to unwrap phase output values.
   * @param totalSensitFlag true to use stage 0 (total) sensitivity;
   * false to use computed sensitivity.
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,
            RespProcessor respProcObj,int outUnitsConvIdx,double [] freqArr,
               boolean logSpacingFlag,boolean verboseFlag,int startStageNum,
                int stopStageNum,boolean useDelayFlag,boolean showInputFlag,
                        boolean listInterpOutFlag, boolean listInterpInFlag,
                           double listInterpTension,boolean unwrapPhaseFlag,
                                  boolean totalSensitFlag,int respTypeIndex,
                                       boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,respProcObj,outUnitsConvIdx,freqArr,logSpacingFlag,
                        verboseFlag,startStageNum,stopStageNum,useDelayFlag,
                           showInputFlag,listInterpOutFlag,listInterpInFlag,
                          listInterpTension,unwrapPhaseFlag,totalSensitFlag,
                                         0.0,respTypeIndex,stdioFlag,outStm,
                                                 plotDisplayOptionPanelObj);
  }

  /**
   * Constructs a response processing callback object.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param respProcObj the "RespProcessor' object being used.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param listInterpOutFlag true to interpolate amp/phase output
   * from responses containing List blockettes.
   * @param listInterpInFlag true to interpolate amp/phase input from
   * List blockettes in responses (before output is calculated).
   * @param listInterpTension tension value for List-blockette
   * interpolation algorithm.
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,
            RespProcessor respProcObj,int outUnitsConvIdx,double [] freqArr,
               boolean logSpacingFlag,boolean verboseFlag,int startStageNum,
                int stopStageNum,boolean useDelayFlag,boolean showInputFlag,
                        boolean listInterpOutFlag, boolean listInterpInFlag,
                                 double listInterpTension,int respTypeIndex,
                                       boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,respProcObj,outUnitsConvIdx,freqArr,logSpacingFlag,
                        verboseFlag,startStageNum,stopStageNum,useDelayFlag,
                           showInputFlag,listInterpOutFlag,listInterpInFlag,
                            listInterpTension,false,false,0.0,respTypeIndex,
                                stdioFlag,outStm,plotDisplayOptionPanelObj);
  }

  /**
   * Constructs a response processing callback object.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param respProcObj the "RespProcessor' object being used.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,
            RespProcessor respProcObj,int outUnitsConvIdx,double [] freqArr,
               boolean logSpacingFlag,boolean verboseFlag,int startStageNum,
                int stopStageNum,boolean useDelayFlag,boolean showInputFlag,
                     int respTypeIndex,boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,respProcObj,outUnitsConvIdx,freqArr,logSpacingFlag,
                        verboseFlag,startStageNum,stopStageNum,useDelayFlag,
                showInputFlag,false,false,0.0,false,false,0.0,respTypeIndex,
                                stdioFlag,outStm,plotDisplayOptionPanelObj);
  }

  /**
   * Constructs a response processing callback object.  After using this
   * constructor, the 'setRespProcObj()' must be called to setup the
   * 'RespProcessor' object before this object is used.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param listInterpOutFlag true to interpolate amp/phase output
   * from responses containing List blockettes.
   * @param listInterpInFlag true to interpolate amp/phase input from
   * List blockettes in responses (before output is calculated).
   * @param listInterpTension tension value for List-blockette
   * interpolation algorithm.
   * @param unwrapPhaseFlag true to unwrap phase output values.
   * @param totalSensitFlag true to use stage 0 (total) sensitivity;
   * false to use computed sensitivity.
   * @param b62XValue sample value for polynomial blockette (62).
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,int outUnitsConvIdx,
               double [] freqArr,boolean logSpacingFlag,boolean verboseFlag,
                    int startStageNum,int stopStageNum,boolean useDelayFlag,
                            boolean showInputFlag,boolean listInterpOutFlag,
                          boolean listInterpInFlag,double listInterpTension,
                            boolean unwrapPhaseFlag,boolean totalSensitFlag,
                                         double b62XValue,int respTypeIndex,
                                       boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,null,outUnitsConvIdx,freqArr,logSpacingFlag,
                        verboseFlag,startStageNum,stopStageNum,useDelayFlag,
                           showInputFlag,listInterpOutFlag,listInterpInFlag,
                listInterpTension,unwrapPhaseFlag,totalSensitFlag,b62XValue,
                  respTypeIndex,stdioFlag,outStm,plotDisplayOptionPanelObj);
  }

  /**
   * Constructs a response processing callback object.  After using this
   * constructor, the 'setRespProcObj()' must be called to setup the
   * 'RespProcessor' object before this object is used.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param listInterpOutFlag true to interpolate amp/phase output
   * from responses containing List blockettes.
   * @param listInterpInFlag true to interpolate amp/phase input from
   * List blockettes in responses (before output is calculated).
   * @param listInterpTension tension value for List-blockette
   * interpolation algorithm.
   * @param unwrapPhaseFlag true to unwrap phase output values.
   * @param totalSensitFlag true to use stage 0 (total) sensitivity;
   * false to use computed sensitivity.
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,int outUnitsConvIdx,
               double [] freqArr,boolean logSpacingFlag,boolean verboseFlag,
                    int startStageNum,int stopStageNum,boolean useDelayFlag,
                            boolean showInputFlag,boolean listInterpOutFlag,
                          boolean listInterpInFlag,double listInterpTension,
                            boolean unwrapPhaseFlag,boolean totalSensitFlag,
                     int respTypeIndex,boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,null,outUnitsConvIdx,freqArr,logSpacingFlag,
                        verboseFlag,startStageNum,stopStageNum,useDelayFlag,
                           showInputFlag,listInterpOutFlag,listInterpInFlag,
                      listInterpTension,unwrapPhaseFlag,totalSensitFlag,0.0,
                  respTypeIndex,stdioFlag,outStm,plotDisplayOptionPanelObj);
  }

  /**
   * Constructs a response processing callback object.  After using this
   * constructor, the 'setRespProcObj()' must be called to setup the
   * 'RespProcessor' object before this object is used.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param listInterpOutFlag true to interpolate amp/phase output
   * from responses containing List blockettes.
   * @param listInterpInFlag true to interpolate amp/phase input from
   * List blockettes in responses (before output is calculated).
   * @param listInterpTension tension value for List-blockette
   * interpolation algorithm.
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,int outUnitsConvIdx,
               double [] freqArr,boolean logSpacingFlag,boolean verboseFlag,
                    int startStageNum,int stopStageNum,boolean useDelayFlag,
                            boolean showInputFlag,boolean listInterpOutFlag,
                          boolean listInterpInFlag,double listInterpTension,
                     int respTypeIndex,boolean stdioFlag,PrintStream outStm,
                           PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,null,outUnitsConvIdx,freqArr,logSpacingFlag,
          verboseFlag,startStageNum,stopStageNum,useDelayFlag,showInputFlag,
           listInterpOutFlag,listInterpInFlag,listInterpTension,false,false,
              0.0,respTypeIndex,stdioFlag,outStm,plotDisplayOptionPanelObj);
  }

  /**
   * Constructs a response processing callback object.  After using this
   * constructor, the 'setRespProcObj()' must be called to setup the
   * 'RespProcessor' object before this object is used.
   * @param tabbedPaneObj the tabbed pane host object.
   * @param outUnitsConvIdx output units conversion index for the
   * requested output units type; one of the '..._UNIT_CONV' values.
   * @param freqArr an array of frequency values to use.
   * @param logSpacingFlag true for log frequency spacing.
   * @param verboseFlag true for verbose output messages.
   * @param startStageNum if greater than zero then the start of the
   * range of stage sequence numbers to use, otherwise all stages
   * are used.
   * @param stopStageNum if greater than zero then the end of the
   * range of stage sequence numbers to use, otherwise only the single
   * stage specified by 'startStageNum' is used.
   * @param useDelayFlag true to use estimated delay in phase calculation.
   * @param showInputFlag true to show RESP input text (sent to console).
   * @param respTypeIndex index value indicating type of output (separate
   * amp/phase file, complex-spectra file or single amp/phase file).
   * @param stdioFlag true for output to 'stdout', false for output
   * to file(s).
   * @param outStm output stream for messages to be sent to (usually
   * 'System.err'), or null for no messages.
   * @param plotDisplayOptionPanelObj plot display option panel.
   */
  public CallbackProcPlot(JTabbedPane tabbedPaneObj,int outUnitsConvIdx,
               double [] freqArr,boolean logSpacingFlag,boolean verboseFlag,
                    int startStageNum,int stopStageNum,boolean useDelayFlag,
                  boolean showInputFlag,int respTypeIndex,boolean stdioFlag,
        PrintStream outStm,PlotDisplayOptionPanel plotDisplayOptionPanelObj)
  {
    this(tabbedPaneObj,null,outUnitsConvIdx,freqArr,logSpacingFlag,
          verboseFlag,startStageNum,stopStageNum,useDelayFlag,showInputFlag,
                  respTypeIndex,stdioFlag,outStm,plotDisplayOptionPanelObj);
  }

  /**
   * Generates the output.
   * @param outGenObj the 'OutputGenerator' object to use.
   * @param channelIdObj the channel ID object to use.
   * @param respEndDateObj end date object to use.
   * @param channelIdFName the output filename to use.
   * @return true if successful, false if error.
   */
  protected boolean generateOutput(final OutputGenerator outGenObj,
                     final ChannelId channelIdObj,final Date respEndDateObj,
                                                final String channelIdFName)
  {
    if(SwingUtilities.isEventDispatchThread())
    {    //called via event-dispatch thread; call generate method directly
      return doGenerateOutput(outGenObj,channelIdObj,respEndDateObj,
                                                            channelIdFName);
    }
    else
    {    //not called via event-dispatch thread; initialize flags
      generateCompletedFlag = generateResultFlag = false;
      SwingUtilities.invokeLater(new Runnable()
          {        //create plot via event-dispatch thread
            public void run()
            {
              generateResultFlag = doGenerateOutput(
                      outGenObj,channelIdObj,respEndDateObj,channelIdFName);
              generateCompletedFlag = true;      //indicate completed
            }
          });
      int count = 0;
      while(!generateCompletedFlag && ++count < 1000)
      {  //wait while plot generate not complete (up to 10 seconds)
        try { Thread.sleep(10); }
        catch(InterruptedException ex) {}
      }
      return generateResultFlag;
    }
  }

  /**
   * Generates the output.
   * @param outGenObj the 'OutputGenerator' object to use.
   * @param channelIdObj the channel ID object to use.
   * @param respEndDateObj end date object to use.
   * @param channelIdFName the output filename to use.
   * @return true if successful, false if error.
   */
  protected boolean doGenerateOutput(OutputGenerator outGenObj,
           ChannelId channelIdObj,Date respEndDateObj,String channelIdFName)
  {
    try
    {
      if(tabbedPaneObj == null)
      {
        outStmPrintln("Internal error:  'tabbedPaneObj' null in " +
                      "'CallbackProcPlot.generateOutout()'");
        return false;
      }
      if(verboseFlag)     //if verbose mode then show message
        outStmPrintln("  Generating output plot");

              //create copy of panel object to prevent local changes
              // from affecting global panel on 'RespEntryPanel':
      final PlotDisplayOptionPanel plotDisplayOptionPanelObj =
           new PlotDisplayOptionPanel(this.globalPlotDisplayOptionPanelObj);

      if(plotDisplayOptionPanelObj.getLogarithmicCheckBox().isSelected() &&
                                       (AmpPhaseComboBox.isShowStagesSelIdx(
        plotDisplayOptionPanelObj.getAmpPhaseComboBox().getSelectedIndex())?
                                   outGenObj.allStagesAnyAmpsNotPositive() :
                                            outGenObj.anyAmpsNotPositive()))
      {  //logarithmic amp axis selected and there are amp values <= 0.0
        outStmPrintln("Amplitude data contains non-positive value(s); " +
                                      "switching to linear-amplitude plot");
        plotDisplayOptionPanelObj.getLogarithmicCheckBox().setSelected(
                                                                     false);
      }

         //create plot panel (pass along tabbed pane and
         // output generator objects, ID object, ID name
         // string, horizontal log flag and plot-options panel):
      final RespPlotPanel plotPanelObj = new RespPlotPanel(tabbedPaneObj,
          outGenObj,channelIdObj,respEndDateObj,channelIdFName,
          logSpacingFlag,plotDisplayOptionPanelObj);

      String tabNameStr = channelIdFName;
      if(tabbedPaneObj.indexOfTab(tabNameStr) >= 0)
      {  //a tab exists with the same name, so a number
         // will be appended to the name:
        int cnt;     //get last character of name:
//        final char endChar = ((cnt=tabNameStr.length()) > 0) ?
//                             tabNameStr.charAt(cnt-1) : '\0';
        cnt = 1;          //initialize counter
        do
        {  //for each count; create a tab string to try:
          tabNameStr = channelIdFName + "_" + (++cnt);
        }  //loop if tab name found (and not too many tries):
        while(tabbedPaneObj.indexOfTab(tabNameStr) >= 0 && cnt < 999);
      }
      //create new tab holding plot panel:
      tabbedPaneObj.addTab(tabNameStr,plotPanelObj);
      //make new tab be the one that's displayed:
      tabbedPaneObj.setSelectedComponent(plotPanelObj);
      return true;
    }
    catch(Exception ex)
    {    //some kind of exception error; show error message
      outStmPrintln("Error generating plot:");
      outStmPrintln(UtilFns.getStackTraceString(ex));
      return false;
    }
  }

  /**
   * Generates the output in text format.
   * @param outGenObj the 'OutputGenerator' object to use.
   * @param channelIdFName the output filename to use.
   * @return true if successful, false if error.
   */
/*
  protected boolean generateTextOutput(OutputGenerator outGenObj,
                                                      String channelIdFName)
  {
    if(tabbedPaneObj == null)
    {
      outStmPrintln("Internal error:  'tabbedPaneObj' null in " +
                                     "'CallbackProcPlot.generateOutout()'");
      return false;
    }
    final JTextArea textAreaObj = new JTextArea();    //create text area
    //put empty-space border around text area for margin:
    textAreaObj.setBorder(BorderFactory.createEmptyBorder(2,5,2,5));
    //wrap text area in scroll pane:
    final JScrollPane scrollPaneObj = new JScrollPane(textAreaObj);
    //create new tab holding text-area scroll pane:
    tabbedPaneObj.addTab(channelIdFName,scrollPaneObj);
    //make new tab be the one that's displayed:
    tabbedPaneObj.setSelectedComponent(scrollPaneObj);
    //generate output and send to text area via stream:
    return respProcObj.outputData(outGenObj,channelIdFName,respTypeIndex,
                    new PrintStream(new TextAreaOutputStream(textAreaObj)));
  }
  */
}
