//CmdLnPlotter.java:  Extension to JEvalResp that accepts command-line
//                    arguments and plots the results.
//
//  5/14/2002 -- [ET]  Initial release version.
//  7/10/2002 -- [ET]  Changed 'apOutputFlag' to 'respTypeIndex'.
//   4/1/2005 -- [ET]  Added support for 'useDelay' and 'showInputFlag'
//                     parameters; increased program frame height.
//  9/23/2005 -- [ET]  Added support for reading input from AMP/PHASE
//                     files ('inAmpPhaseFlag'/'if' parameter).
//   6/1/2010 -- [ET]  Added parameters to 'new CallbackProcPlot()'
//                     invocations:  listInterpOutFlag, listInterpInFlag,
//                     listInterpTension, unwrapPhaseFlag, totalSensitFlag.
//   4/2/2012 -- [ET]  Added support for specification of web-services
//                     server.
// 10/29/2013 -- [ET]  Added support for 'b62_x' parameter.
//

package com.isti.jplotresp;

import java.io.PrintStream;
import java.util.Vector;
import java.util.Iterator;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.*;
import com.isti.util.CfgPropItem;
import com.isti.util.gui.TextAreaOutputStream;
import com.isti.jevalresp.RespUtils;
import com.isti.jevalresp.AmpPhaseBlk;
import com.isti.jevalresp.NetVersionFlag;
import com.isti.jevalresp.OutputGenerator;
import com.isti.jevalresp.Run;


/**
 * Class CmdLnPlotter is an extension to JEvalResp that accepts command-line
 * arguments and plots the results.
 */
public class CmdLnPlotter extends Run
{
    //make the "look & feel" setup be the first thing called:
  private SetupLookAndFeel lafObj = new SetupLookAndFeel();
  protected final JPanel mainPanel = new JPanel(new BorderLayout());
  protected final JTabbedPane tabbedPaneObj = new JTabbedPane();
  protected final JTextArea textAreaObj = new JTextArea();

         /** Flag set true to read input from AMP/PHASE files. */
  protected final CfgPropItem inAmpPhaseFlagProp =
                              paramProps.add("inAmpPhaseFlag",Boolean.FALSE,
                                    "if","Read input from AMP/PHASE files");

  /**
   * Creates the panels for the GUI.
   * @return the main panel object, or null if error occurred (in which
   * case an error message will be sent to 'stderr').
   */
  public JPanel createProgramPanels()
  {
    try
    {
         //put empty-space border around text area for margin:
      textAreaObj.setBorder(BorderFactory.createEmptyBorder(2,5,2,5));
         //add text area in scroll pane into new tab:
      tabbedPaneObj.addTab("Console",new JScrollPane(textAreaObj));
         //add tabbed pane to main panel:
      mainPanel.add(tabbedPaneObj,BorderLayout.CENTER);
    }
    catch(Exception ex)
    {
      System.err.println("Error creating program panels:  " + ex);
      return null;
    }
    return mainPanel;
  }

  /**
   * Generates the program output.
   * @param args program args
   */
  public void generateProgramOutput(final String [] args)
  {
    SwingUtilities.invokeLater(new Runnable()
        {     //execute via event-dispatch thread:
          public void run()
          {
              //create TextAreaOutputStream and wrap PrintStream around it
              // //(enable auto-flush):
            final PrintStream textStmObj = new PrintStream(
                                new TextAreaOutputStream(textAreaObj),true);
              //save current 'stdout' and 'stderr' stream objects:
            final PrintStream sysOutSaveStream = System.out;
            final PrintStream sysErrSaveStream = System.err;
            System.setOut(textStmObj);      //redirect to text area streams
            System.setErr(textStmObj);

              //process parameters and generate output:
            if(!processAndOutput(args))                    //if error then
              System.err.println(getErrorMessage());       //show message

            System.setOut(sysOutSaveStream);     //restore orig sys streams
            System.setErr(sysErrSaveStream);
            if(textStmObj.checkError())     //if stream errors then show msg
              System.err.println("Error sending output to text area stream");
            textStmObj.close();             //close text area output stream
          }
        });
  }

  /**
   * Processes parameters, performs requested operations and generates
   * the output.
   * @param args an array of command-line parameters.
   * @return true if successful, false if a parameter error was detected
   * (in which case the 'getErrorMessage()' and 'getExitStatusValue()'
   *  methods may be used to fetch information about the error.)
   */
  public boolean processAndOutput(String [] args)
  {
         //load and preprocess command-line parameters:
    final Vector paramsVec;
    if((paramsVec=preprocessParameters(args)) == null)
      return true;      //if null returned then exit program

    final boolean inAmpPhaseFlag = inAmpPhaseFlagProp.booleanValue();
         //if not reading AMP/PHASE then process non-switch parameters:
    if(!inAmpPhaseFlag && !processNonSwitchParams(paramsVec))
      return false;
         //process switch parameters:
    if(!processSwitchParams())
      return false;

    if(!inAmpPhaseFlag)
    {    //not reading input from AMP/PHASE files
      String str;            //check frequency array parameters:
      if((str=RespUtils.checkFreqArrayParams(minFreqValue,maxFreqValue,
                                       numberFreqs,logSpacingFlag)) != null)
      {  //frequency array parameter invalid
        setErrorMessage(str);       //setup error message
//        exitStatusValue = 15;       //setup exit status error code
        return false;
      }
         //generate array of frequency values:
      if((frequenciesArray=RespUtils.generateFreqArray(minFreqValue,
                          maxFreqValue,numberFreqs,logSpacingFlag)) == null)
      {   //error generating frequency array (shouldn't happen); set msg
        setErrorMessage("Error generating frequency array");
//        exitStatusValue = 15;       //setup exit status error code
        return false;
      }

         //create callback object that plots output:
      final CallbackProcPlot procPlotObj = new CallbackProcPlot(
              tabbedPaneObj,outUnitsConvIdx,frequenciesArray,logSpacingFlag,
          verboseFlag,startStageNum,stopStageNum,useDelayFlag,showInputFlag,
                       listInterpOutFlag,listInterpInFlag,listInterpTension,
          unwrapPhaseFlag,totalSensitFlag,b62XValue,respTypeIndex,stdioFlag,
                                   System.err,new PlotDisplayOptionPanel());

         //find responses and output results to plots:
      final boolean resultFlag;
      if(NetVersionFlag.value && propsFileString != null &&
                                               propsFileString.length() > 0)
      {  //"network" version of program and server props filename was given
              //check if web-services server specified:
        if(checkWebServicesServer())
          resultFlag = generateWebResponses(procPlotObj);
        else  //if not web services then fetch from FISSURES network server:
          resultFlag = generateNetResponses(procPlotObj); //generate responses
      }
      else
        resultFlag = generateResponses(procPlotObj);    //generate responses
      return resultFlag;
    }
    else
    {    //reading input from AMP/PHASE files
      if(paramsVec.size() <= 0)
      {  //no filename parameters found on command line; setup error message
        setErrorMessage("No AMP/PHASE input files specified");
        return false;
      }
      Iterator iterObj = paramsVec.iterator();
      final Vector freqValsSetVec = new Vector();
      Object obj;
      FreqValuesSet setObj;
      while(iterObj.hasNext())
      {  //for each AMP/PHASE input file parameter on command line
        if((obj=iterObj.next()) instanceof String)
        {     //filename string fetched OK
          if(verboseFlag)
          {   //verbose flag set; show message
            System.err.println("Processing AMP/PHASE input file \"" +
                                                                obj + "\"");
          }
          setObj = new FreqValuesSet((String)obj);
          if(!setObj.readFileData())        //read input file data
          {   //error processing file; setup error message
            setErrorMessage("Error processing \"" + obj + "\":  " +
                                            setObj.getErrorMessageString());
            return false;
          }
          freqValsSetVec.add(setObj);       //add set to vector
        }
      }
      iterObj = freqValsSetVec.iterator();
      FreqValuesSet nextSetObj;
      AmpPhaseBlk [] ampPhaseBlkArr;
      String idFNameStr;
      int ampPhaseSelIdx;
      PlotDisplayOptionPanel optionPanelObj;
      CallbackProcPlot procPlotObj;
      OutputGenerator outGenObj;
      if(iterObj.hasNext())
      {  //next set available
        nextSetObj = ((obj=iterObj.next()) instanceof FreqValuesSet) ?
                                                  (FreqValuesSet)obj : null;
      }
      else    //next set not available
        nextSetObj = null;
      while(nextSetObj != null)
      {  //for each freq-values set (or pair)
        setObj = nextSetObj;
        idFNameStr = setObj.getIDFNameStr();     //setup "ID" file name
        if(iterObj.hasNext())
        {     //next set available; fetch it
          nextSetObj = ((obj=iterObj.next()) instanceof FreqValuesSet) ?
                                                  (FreqValuesSet)obj : null;
        }
        else  //next set not available
          nextSetObj = null;
              //check if next set should be used to make AMP/PHASE pair:
        if(nextSetObj != null && !setObj.containsSecondValues() &&
                                       !nextSetObj.containsSecondValues() &&
                                        setObj.isFreqArrayEqual(nextSetObj))
        {     //sets do not contain both amp & phase values
              // and sets have same frequency array values
                   //use sets to build array of 'AmpPhaseBlk' objects
                   // (if next file starts with "AMP" and current
                   //  file starts with "PHASE" then swap order):
          if(nextSetObj.isAmpFileName() && setObj.isPhaseFileName())
          {
            ampPhaseBlkArr =      //build array of 'AmpPhaseBlk' objects:
                     FreqValuesSet.buildAmpPhaseBlkArray(nextSetObj,setObj);
                                  //add next file name to "ID" string:
            idFNameStr = nextSetObj.getIDFNameStr() + '/' + idFNameStr;
          }
          else
          {
            ampPhaseBlkArr =      //build array of 'AmpPhaseBlk' objects:
                     FreqValuesSet.buildAmpPhaseBlkArray(setObj,nextSetObj);
                                  //add next file name to "ID" string:
            idFNameStr += '/' + nextSetObj.getIDFNameStr();
          }
                   //setup to display both amp and phase plots:
          ampPhaseSelIdx = AmpPhaseComboBox.AMP_PHASE_IDX;
              //attempt to fetch next set
          if(iterObj.hasNext())
          {   //next set available; fetch it
            nextSetObj = ((obj=iterObj.next()) instanceof FreqValuesSet) ?
                                                  (FreqValuesSet)obj : null;
          }
          else     //next set not available
            nextSetObj = null;
        }
        else  //at least one set contains both amp & phase values
        {     // or sets do not have same frequency array values
                   //create array of 'AmpPhaseBlk' object from current set:
          ampPhaseBlkArr = setObj.createAmpPhaseBlkArray();
                   //if set contains both amp and phase values then
                   // setup to display both amp and phase plots:
          if(setObj.containsSecondValues())
            ampPhaseSelIdx = AmpPhaseComboBox.AMP_PHASE_IDX;
          else     //set contains only amp or phase; setup
          {        // plot display based on file name:
            ampPhaseSelIdx = setObj.isPhaseFileName() ?
                      AmpPhaseComboBox.PHASE_IDX : AmpPhaseComboBox.AMP_IDX;
          }
        }
                   //create array of frequency values from current set:
        frequenciesArray = setObj.createFrequenciesArray();
                   //if freq spacing not set via command-line parameter
                   // then determine spacing from frequency values:
        if(!typeOfSpacingProp.getCmdLnLoadedFlag())
          logSpacingFlag = setObj.isFreqLogSpacing();

//        System.out.println();
//        System.out.println(idFNameStr + ":");
//        for(int i=0; i<ampPhaseBlkArr.length; ++i)
//        {
//          System.out.println("  freq=" + frequenciesArray[i] + ", " +
//                                                         ampPhaseBlkArr[i]);
//        }
//        System.out.println(idFNameStr + " size = " + ampPhaseBlkArr.length);

              //create plot-display options panel object:
        optionPanelObj = new PlotDisplayOptionPanel(ampPhaseSelIdx,
                                         ConfigManager.DEFAULT_LOG_AMP_FLAG,
                                     ConfigManager.DEFAULT_SHOW_POINTS_FLAG,
                              ConfigManager.DEFAULT_COMBINE_AMP_PHASE_FLAG);

              //create callback object that plots output:
        procPlotObj = new CallbackProcPlot(tabbedPaneObj,
                         OutputGenerator.DEFAULT_UNIT_CONV,frequenciesArray,
                      logSpacingFlag,verboseFlag,startStageNum,stopStageNum,
                               useDelayFlag,showInputFlag,listInterpOutFlag,
                         listInterpInFlag,listInterpTension,unwrapPhaseFlag,
                                    totalSensitFlag,b62XValue,respTypeIndex,
                                       stdioFlag,System.err,optionPanelObj);
              //create 'OutputGenerator' obj pre-loaded with amp/phase data:
        outGenObj = new OutputGenerator(ampPhaseBlkArr,frequenciesArray,
                                                            logSpacingFlag);
              //generate output plot:
        procPlotObj.generateOutput(outGenObj,null,null,idFNameStr);
      }
      return true;
    }
  }

  /**
   * Program entry point; creates a 'CmdLnPlotter' and shows its main
   * panel in a frame.
   * @param args program arguments
   */
  public static void main(String [] args)
  {
    final JFrame mainFrame = new JFrame();       //create frame; set title
    mainFrame.setTitle("JEvalRespPlot using " + Run.REVISION_STR);
    mainFrame.setSize(800,675);        //set frame size
         //setup to exit on window close
    mainFrame.addWindowListener(new WindowAdapter()
        {
          public void windowClosing(WindowEvent e)
          {
            System.exit(0);             //exit application
          }
        });
         //get screen size:
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
         //put frame at center of screen:
    mainFrame.setLocation((d.width-mainFrame.getSize().width)/2,
                          (d.height-mainFrame.getSize().height)/2);
         //create command-line plotter object:
    final CmdLnPlotter plotterObj = new CmdLnPlotter();
    final JPanel mainPanel;
    if((mainPanel=plotterObj.createProgramPanels()) != null)
    {    //panels created OK; get content pane for frame
      final Container contentPane = mainFrame.getContentPane();
      contentPane.setLayout(new BorderLayout());       //set layout
      contentPane.add(mainPanel,BorderLayout.CENTER);  //add main panel
      mainFrame.setVisible(true);                //make frame visible
      plotterObj.generateProgramOutput(args);    //generate/display output
    }
  }

  /**
   * Class SetupLookAndFeel has a constructor that sets up the "look &
   * feel" for the applet.  Creating an object of this class at the top
   * of a class definition causes all objects created afterward to have
   * to desired "look & feel".  (Putting the 'UIManager.setLookAndFeel()'
   * call into a 'static' block only works the first time an applet is
   * started in a given browser.)
   */
  protected class SetupLookAndFeel
  {
    public SetupLookAndFeel()
    {
      try
      {
         //set "look & feel" to be that of host system:
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
//        UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
      }
      catch (Exception e) {}
    }
  }
}
