//ConfigManager.java:  Manages an XML-format configuration file.
//
//   3/5/2003 -- [KF]  Initial version.
//  5/13/2003 -- [ET]  Took "default" out of configuration property names
//                     and item objects.
//  5/28/2003 -- [KF]  Added persistence of entry-screen settings.
//  5/29/2003 -- [KF]  Changed default on "Remember Settings" to 'true';
//                     added 'setAllItemsToDefaults()' method.
//  4/11/2005 -- [ET]  Added 'useDelay' and 'showInput' items.
// 10/31/2005 -- [ET]  Added items for List-blockette interpolation.
//  7/16/2009 -- [KF]  Added 'useIstiFileChooserFlagProp' item.
//  5/26/2010 -- [ET]  Added 'unwrapPhaseFlag' and 'totalSensitFlag' items.
//   6/7/2010 -- [ET]  Added 'showFreqLineFlag' item.
//  1/30/2012 -- [ET]  Changed default for 'serverFile' parameter to
//                     "irisws.prop".
//   4/2/2012 -- [ET]  Added 'multiSvrFlag' item.
// 10/29/2013 -- [ET]  Added 'b62XValue' item.
//

package com.isti.jplotresp;

import java.io.*;
import java.util.List;
import java.util.Hashtable;
import java.util.Map;
import java.util.Iterator;

import java.awt.Component;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.AbstractButton;
import javax.swing.JComboBox;
import javax.swing.text.JTextComponent;

import org.jdom.Element;
import com.isti.util.*;
import com.isti.jevalresp.Run;

/**
 * Class ConfigManager manages an XML-format configuration file.
 */
public class ConfigManager
{
  // empty text
  public static final String EMPTY_TEXT = "";
  // Value for config tag
  public static final String CONFIG = "RespConfig";
  // Value for "Settings" tag
  public static final String SETTINGS = "Settings";
  // Config group selection object
  public static final String CONFIG_GROUP = "Config";

  protected final String cfgFileNameStr;     //handle for cfg file name
  protected final Element configRootElement;     //root element in file
  protected final IstiXmlUtils istiXmlUtilsObj;  //XML utils object
  protected String errorMessage = null;     //handle for error message

  /** Configuration properties object for Settings. */
  public final CfgProperties settingsProps = new CfgProperties();

  //components table for settings
  private final Hashtable componentsTable = new Hashtable();
  //components table for settings
  private final Hashtable saveComponentsTable = new Hashtable();

  /** Network. */
  public final CfgPropItem networkProp =
      settingsProps.add("network", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** Station. */
  public final CfgPropItem stationProp =
      settingsProps.add("station", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** Location. */
  public final CfgPropItem locationProp =
      settingsProps.add("location", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** Channel. */
  public final CfgPropItem channelProp =
      settingsProps.add("channel", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** Minimum frequency. */
  public final CfgPropItem minFreqProp =
      settingsProps.add("minFreq", new Double(
      0.0001)).setGroupSelObj(CONFIG_GROUP);

  /** Maximum frequency. */
  public final CfgPropItem maxFreqProp =
      settingsProps.add("maxFreq", new Double(
      100.0)).setGroupSelObj(CONFIG_GROUP);

  /** Number of frequencies. */
  public final CfgPropItem numFreqProp =
      settingsProps.add("numFreq", new Integer(
      100)).setGroupSelObj(CONFIG_GROUP);

  /** Begin time year. */
  public final CfgPropItem beginTimeYearProp =
      settingsProps.add("beginTimeYear",
      new String()).setGroupSelObj(CONFIG);

  /** Begin time julian day. */
  public final CfgPropItem beginTimeJulianDayProp =
      settingsProps.add("beginTimeJulianDay",
      new String()).setGroupSelObj(CONFIG);

  /** Begin time time. */
  public final CfgPropItem beginTimeTimeProp =
      settingsProps.add("beginTimeTime",
      new String()).setGroupSelObj(CONFIG);

  /** End time year. */
  public final CfgPropItem endTimeYearProp =
      settingsProps.add("endTimeYear",
      new String()).setGroupSelObj(CONFIG);

  /** End time julian day. */
  public final CfgPropItem endTimeJulianDayProp =
      settingsProps.add("endTimeJulianDay",
      new String()).setGroupSelObj(CONFIG);

  /** End time time. */
  public final CfgPropItem endTimeTimeProp =
      settingsProps.add("endTimeTime",
      new String()).setGroupSelObj(CONFIG);

  /** Remember Settings */
  public final CfgPropItem rememberSettingsProp =
      settingsProps.add("rememberSettings", Boolean.TRUE
      ).setGroupSelObj(CONFIG_GROUP);

  /** Remember Settings */
  public final CfgPropItem multiOutProp =
      settingsProps.add("multiOut", Boolean.FALSE
      ).setGroupSelObj(CONFIG_GROUP);

  /** Filename selected flag. */
  public final CfgPropItem filenameSelectedFlagProp =
      settingsProps.add("filenameSelectedFlag",
      Boolean.TRUE).setGroupSelObj(CONFIG_GROUP);

  /** Filename. */
  public final CfgPropItem filenameProp =
      settingsProps.add("filename", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** Server. */
  public final CfgPropItem serverFileProp =
      settingsProps.add("serverFile", new String(
      "irisws.prop")).setGroupSelObj(CONFIG_GROUP);

  /** Start stage. */
  public final CfgPropItem startStageProp =
      settingsProps.add("startStage", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** End stage. */
  public final CfgPropItem endStageProp =
      settingsProps.add("endStage", EMPTY_TEXT).setGroupSelObj(CONFIG_GROUP);

  /** Enable verbose output. */
  public final CfgPropItem verboseProp =
      settingsProps.add("verbose", Boolean.TRUE).setGroupSelObj(CONFIG_GROUP);

  /** Enable headers in output files. */
  public final CfgPropItem headersProp =
      settingsProps.add("headers", Boolean.TRUE).setGroupSelObj(CONFIG_GROUP);

  /** Enable use of estimated delay in phase calculation. */
  public final CfgPropItem useDelayProp =
      settingsProps.add("useDelay", Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Flag set true to fetch from all specified web-services servers. */
  public final CfgPropItem multiSvrFlagProp =
      settingsProps.add("multiSvrFlag",Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Flag set true to unwrap phase output values. */
  public final CfgPropItem unwrapPhaseFlagProp =
      settingsProps.add("unwrapPhaseFlag",Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Flag set true to use stage 0 sensitivity instead of computed. */
  public final CfgPropItem totalSensitFlagProp =
      settingsProps.add("totalSensitFlag",Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Enable showing of RESP input text (sent to console). */
  public final CfgPropItem showInputProp =
      settingsProps.add("showInput", Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Sample value for polynomial blockette. */
  public final CfgPropItem b62XValueProp =
      settingsProps.add("b62XValue", EMPTY_TEXT).
      setGroupSelObj(CONFIG_GROUP);

  /** Enable interpolation of List blockette output. */
  public final CfgPropItem interpOutputProp =
      settingsProps.add("interpOutput", Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Enable interpolation of List blockette input. */
  public final CfgPropItem interpInputProp =
      settingsProps.add("interpInput", Boolean.FALSE).
      setGroupSelObj(CONFIG_GROUP);

  /** Tension for List blockette interpolation. */
  public final CfgPropItem interpTensionProp =
      settingsProps.add("interpTension", new Double(
      Run.INTERP_TENSION_DEFVAL)).setGroupSelObj(CONFIG_GROUP);

  /** Amp/phase selection. */
  public final static int DEFAULT_AMP_PHASE_SEL_IDX =
      AmpPhaseComboBox.DEFAULT_IDX;
  public final CfgPropItem ampPhaseSelProp =
      settingsProps.add("ampPhaseSel", new Integer(
      DEFAULT_AMP_PHASE_SEL_IDX)).setValidator(0,
      AmpPhaseComboBox.NUM_INDICES - 1).setGroupSelObj(CONFIG_GROUP);

  /** Show points flag. */
  public final static boolean DEFAULT_SHOW_POINTS_FLAG = false;
  public final CfgPropItem showPointsFlagProp =
      settingsProps.add("showPointsFlag",
      new Boolean(DEFAULT_SHOW_POINTS_FLAG));

  /** Log-amplitude flag. */
  public final static boolean DEFAULT_LOG_AMP_FLAG = true;
  public final CfgPropItem logAmpFlagProp =
      settingsProps.add("logAmpFlag", new Boolean(DEFAULT_LOG_AMP_FLAG));

  /** Combine amp/phase flag. */
  public final static boolean DEFAULT_COMBINE_AMP_PHASE_FLAG = false;
  public final CfgPropItem combineAmpPhaseFlagProp =
      settingsProps.add("combineAmpPhaseFlag",
      new Boolean(DEFAULT_COMBINE_AMP_PHASE_FLAG));

  /** Show frequency line flag. */
  public final static boolean DEFAULT_SHOW_FREQ_LINE_FLAG = true;
  public final CfgPropItem showFreqLineFlagProp =
      settingsProps.add("showFreqLineFlag",
      new Boolean(DEFAULT_SHOW_FREQ_LINE_FLAG));

  /** Units. */
  public final CfgPropItem unitsProp =
      settingsProps.add("units", new Integer(
      Run.UNIT_CONV_DEFIDX)).setValidator(0,
      Run.UNIT_CONV_LONGSTRS.length - 1).setGroupSelObj(CONFIG);

  /** Frequency spacing. */
  public final CfgPropItem frequencySpacingProp =
      settingsProps.add("frequencySpacing", new Integer(0)).setValidator(0,
      Run.TYPE_SPACE_LONGSTRS.length - 1).setGroupSelObj(CONFIG);

  /** Response type. */
  public final CfgPropItem responseTypeProp =
      settingsProps.add("responseType", new Integer(0)).setValidator(0,
      Run.RESP_TYPE_LONGSTRS.length - 1).setGroupSelObj(CONFIG);

  /** Output Directory. */
  public final CfgPropItem outputDirectoryProp =
      settingsProps.add("outputDirectory", EMPTY_TEXT).setGroupSelObj(
      CONFIG_GROUP);

  /** Width of program window frame. */
  public final CfgPropItem programFrameWidthProp = settingsProps.add(
      "programFrameWidth",new Integer(PlotRun.DEF_FRAME_WIDTH));

  /** Height of program window frame. */
  public final CfgPropItem programFrameHeightProp = settingsProps.add(
      "programFrameHeight",new Integer(PlotRun.DEF_FRAME_HEIGHT));

  /** X-position of program window frame (<0=centered). */
  public final CfgPropItem programFrameXPosProp = settingsProps.add(
      "programFrameXPos",new Integer(-1));

  /** Y-position of program window frame (<0=centered). */
  public final CfgPropItem programFrameYPosProp = settingsProps.add(
      "programFrameYPos",new Integer(-1));

  /** Use ISTI file choosers flag. */
  public final CfgPropItem useIstiFileChooserFlagProp = settingsProps.add(
      "useIstiFileChooserFlag", Boolean.TRUE).setGroupSelObj(CONFIG_GROUP);

  /**
   * Creates an object for processing XML-format configuration file.
   * The 'getErrorFlag()' method may be used to check if the load was
   * successful and the 'getErrorMessage()' method may be used to
   * retrieve an error message, if any.
   * @param cfgFileNameStr the name of the XML-format input
   * config file to read and process.
   */
  public ConfigManager(String cfgFileNameStr)
  {
    //save filenames
    this.cfgFileNameStr = cfgFileNameStr;

    istiXmlUtilsObj = new IstiXmlUtils();        //create XML utils object

    if ((new File(cfgFileNameStr)).exists())  //config file exists
    {
      //load configuration file into JDOM Element objects:
      if (!istiXmlUtilsObj.loadFile(cfgFileNameStr, CONFIG))
      {
        //error loading; fetch and set error message
        setErrorMessage("Error loading configuration file:  " +
                        istiXmlUtilsObj.getErrorMessage());
        configRootElement = null;        //indicate no data
      }
      else
      {
        configRootElement = istiXmlUtilsObj.getRootElement();
        String str = configRootElement.getText().trim();
        if (str != null && str.length() > 0)
        {
          //non-attribute text data in element
          setErrorMessage("Unexpected text data found in \"" +
                          CONFIG + "\" element in configuration file \"" +
                          cfgFileNameStr + "\":  " + str);
        }
      }
    }
    else      //configuration file does not exist
    {
      configRootElement = null;        //indicate no data
    }
  }

  /**
   * Add the component to the components table.
   * @param comp component to add.
   * @param cfgPropItem the config property editor.
   * @param saveFlag true to save when saving settings.
   * @param clrDefFlag true to restore default if edit value cleared.
   */
  protected void addToComponentsTable(final Component comp,
              CfgPropItem cfgPropItem, boolean saveFlag, boolean clrDefFlag)
  {
    componentsTable.put(comp, cfgPropItem);
    if (saveFlag)
      saveComponentsTable.put(comp, cfgPropItem);

    if(clrDefFlag && comp instanceof JTextComponent)
    {    //restore-on-clear flag set and comp is text component
      comp.addFocusListener(new FocusAdapter()
          {
            public void focusLost(FocusEvent e)
            {
              try
              {
                final JTextComponent textCompObj = (JTextComponent)comp;
                if(textCompObj.getText().length() <= 0)
                {  //text edit field is empty
                  final Object obj;
                  if((obj=componentsTable.get(comp)) instanceof CfgPropItem)
                  {     //cfgProp item for edit field fetched OK; enter def
                    textCompObj.setText(((CfgPropItem)obj).
                                       getDefaultValue().toString().trim());
                  }
                }
              }
              catch(Exception ex)
              {
                System.err.println("Error restoring default value in " +
                                                   "ConfigManager:  " + ex);
                ex.printStackTrace();
              }
            }
          });
    }
  }

  /**
   * Add the component to the components table.
   * @param comp component to add.
   * @param cfgPropItem the config property editor.
   * @param saveFlag true to save when saving settings.
   */
  protected void addToComponentsTable(Component comp,
                                  CfgPropItem cfgPropItem, boolean saveFlag)
  {
    addToComponentsTable(comp,cfgPropItem,saveFlag,false);
  }

  /**
   * Gets the components table.
   * @return the components table.
   */
  protected Map getComponentsTable()
  {
    return componentsTable;
  }

  /**
   * Load the component configuration settings.
   */
  public void loadComponentConfigSettings()
  {
    Iterator it = componentsTable.keySet().iterator();
    while (it.hasNext())
    {
      final Object key = it.next();
      final Object value = componentsTable.get(key);

      if (value instanceof CfgPropItem)
      {
        final CfgPropItem cpi = (CfgPropItem)value;

        //get the setting
        if (key instanceof JTextComponent)
        {
          ((JTextComponent)key).setText(cpi.stringValue().trim());
        }
        else if (key instanceof AbstractButton)
        {
          ((AbstractButton)key).setSelected(cpi.booleanValue());
        }
        else if (key instanceof JComboBox)
        {
          ((JComboBox)key).setSelectedIndex(cpi.intValue());
        }
      }
    }
  }

  /**
   * Save the component configuration settings.
   */
  public void saveComponentConfigSettings()
  {
    Iterator it = saveComponentsTable.keySet().iterator();
    while (it.hasNext())
    {
      final Object key = it.next();
      final Object value = saveComponentsTable.get(key);

      if (value instanceof CfgPropItem)
      {
        final CfgPropItem cpi = (CfgPropItem)value;

        //save the setting
        if (key instanceof JTextComponent)
        {
          cpi.setValueString(((JTextComponent)key).getText().trim());
        }
        else if (key instanceof AbstractButton)
        {
          cpi.setValue(((AbstractButton)key).isSelected());
        }
        else if (key instanceof JComboBox)
        {
          cpi.setValue(((JComboBox)key).getSelectedIndex());
        }
      }
    }
  }

  /**
   * Save the configuration.
   * @return 'true' if successful, false if an error occurred (in which
   * case the 'getErrorMessage()' method may be used to fetch a
   * description of the error).
   */
  public boolean saveConfig()
  {
    istiXmlUtilsObj.clearErrorMessage();    //clear any previous error msg
    final Element configElementObj = new Element(CONFIG);

    //save the settings
    if (!saveSettings(configElementObj))
      return false;

    //write it all out to the file
    if(!istiXmlUtilsObj.saveToFile(cfgFileNameStr, configElementObj))
    {
      //error saving to file; set error message
      setErrorMessage("Error writing configuration file \"" +
                      cfgFileNameStr + "\": " +
                      istiXmlUtilsObj.getErrorMessage());
      return false;          //return error flag
    }
    return true;
  }

  /**
   * Loads the set of "name = value" items specified by the "Settings"
   * element in the configuration file.
   * @return 'true' if successful, false if an error occurred (in which
   * case the 'getErrorMessage()' method may be used to fetch a
   * description of the error).
   */
  public boolean loadSettings()
  {
    return loadSettings(configRootElement);
  }

  /**
   * Loads the set of "name = value" items specified by the "Settings"
   * element in the configuration file.
   * @param rootElement root element.
   * @return 'true' if successful, false if an error occurred (in which
   * case the 'getErrorMessage()' method may be used to fetch a
   * description of the error).
   */
  protected boolean loadSettings(Element rootElement)
  {
    if(getErrorFlag())       //if previous error (from constructor) then
      return false;          //abort method
    if(rootElement == null)      //if root element was not found then
      return true;                     //return OK flag
    //get list of "Settings" elements:
    final List settingsElemsList = rootElement.getChildren(SETTINGS);
    if(settingsElemsList.size() <= 0)
    {
      //no "Settings" elements found; set error message
      setErrorMessage("No \"" + SETTINGS +
                      "\" elements found in configuration file \"" +
                      cfgFileNameStr + "\"");
      return false;               //abort method
    }
    if(settingsElemsList.size() > 1)
    {
      //more than one "Settings" element found; set error message
      setErrorMessage("More than one \"" + SETTINGS +
                      "\" element found in configuration file \"" +
                      cfgFileNameStr + "\"");
      return false;               //abort method
    }
    //fetch "Settings" element object:
    Object obj;
    Element elementObj;
    String elemNameStr = null;
    if(!((obj=settingsElemsList.get(0)) instanceof Element) ||
       !SETTINGS.equals(
       (elemNameStr=((elementObj=(Element)obj).getName()))))
    {
      //unable to fetch Element or name is not "Settings"; set err msg
      setErrorMessage("Error fetching \"" + SETTINGS +
                      "\" element from configuration file \"" + cfgFileNameStr +
                      "\" (found name = \"" + elemNameStr + "\")");
      return false;               //abort method
    }
    //process text inside "Settings" element:
    //fetch text inside element, convert to a byte array and
    // load "name = value" items into 'CfgProperties' object:
    if(!settingsProps.load(new ByteArrayInputStream(
        elementObj.getText().getBytes())))
    {
      //error processing "name = value" items; fetch and set error msg
      setErrorMessage("Error processing \"" + SETTINGS +
                      "\":  " + settingsProps.getErrorMessage());
      return false;               //abort method
    }
    return true;             //return OK flag
  }

  /**
   * Saves the set of "name = value" items specified by the "Settings"
   * element in the configuration file.
   * @param configElementObj configuration element object.
   * @return 'true' if successful, false if an error occurred (in which
   * case the 'getErrorMessage()' method may be used to fetch a
   * description of the error).
   */
  protected boolean saveSettings(Element configElementObj)
  {
    //create byte array stream to receive properties data:
    final ByteArrayOutputStream stmObj = new ByteArrayOutputStream();

    try
    {
      //store properties data in byte array stream
      // (save all props, no header, not cmdln-only):
      settingsProps.store(stmObj,null,false,false,false);
    }
    catch (IOException ioException)
    {
      setErrorMessage("Error storing configuration: " + ioException);
      return false;               //abort method
    }

    //create a "Settings" element
    final Element settingsElementObj = new Element(SETTINGS);

    //set text to settings properties (insert leading newline):
    final String stmObjStr = UtilFns.newline + stmObj.toString();
    settingsElementObj.setText(stmObjStr);

    //put the "Settings" element inside of the config element
    configElementObj.addContent(settingsElementObj);

    return true;             //return OK flag
  }

  /**
   * Sets value on all items to the default value.
   */
  public void setAllItemsToDefaults()
  {
    settingsProps.setAllToDefaults();
  }

  /**
   * Enters an error message (if none previously entered).
   * @param str the error message.
   */
  protected void setErrorMessage(String str)
  {
    if(errorMessage == null)      //if no previous error then
      errorMessage = str;         //set error message
  }

  /**
   * Returns true if an error was detected.  The error message may be
   * fetched via the 'getErrorMessage()' method.
   * @return The flag value.
   */
  public boolean getErrorFlag()
  {
    return (errorMessage != null);
  }

  /**
   * Returns message string for last error (or 'No error' if none).
   * @return The error message.
   */
  public String getErrorMessage()
  {
    return (errorMessage != null) ? errorMessage : "No error";
  }

  /** Clears the error message string. */
  public void clearErrorMessage()
  {
    errorMessage = null;
  }
}
