//PlotDisplayOptionPanel.java:  Panel with plot display options.
//
//  3/19/2003 -- [KF]
//  5/12/2003 -- [ET]  Added tooltips to GUI components.
//  5/28/2003 -- [KF]  Added persistence of entry-screen settings.
//   6/7/2010 -- [ET]  Added "Show Freq Line" item.
//

package com.isti.jplotresp;

import javax.swing.*;

/**
 * Class PlotDisplayOptionPanel is a panel with plot display options.
 */
public class PlotDisplayOptionPanel extends JPanel
{
  private final JPanel ampPhaseComboBoxPanel;         //amp/phase combo box panel
  private final AmpPhaseComboBox ampPhaseComboBox;    //amp/phase combo box
  private final JPanel logarithmicPanel;              //log panel
  private final JCheckBox logarithmicCheckBox;        //log check box
  private final JPanel combineAmpPhasePanel;          //combine amp/phase panel
  private final JCheckBox combineAmpPhaseCheckBox;    //combine amp/phase check box
  private final JPanel showPointsPanel;               //show points panel
  private final JCheckBox showPointsCheckBox;         //show points check box
  private final JPanel showFreqLinePanel;             //show freq line panel
  private final JCheckBox showFreqLineCheckBox;       //show freq line check box

  /**
   * @return the amp/phase combo box.
   */
  public AmpPhaseComboBox getAmpPhaseComboBox()
  {
    return ampPhaseComboBox;
  }

  /**
   * @return the logarithmic check box.
   */
  public JCheckBox getLogarithmicCheckBox()
  {
    return logarithmicCheckBox;
  }

  /**
   * @return the combine amp/phase check box.
   */
  public JCheckBox getCombineAmpPhaseCheckBox()
  {
    return combineAmpPhaseCheckBox;
  }

  /**
   * @return the show points check box.
   */
  public JCheckBox getShowPointsCheckBox()
  {
    return showPointsCheckBox;
  }

  /**
   * @return the show frequency line check box.
   */
  public JCheckBox getShowFreqLineCheckBox()
  {
    return showFreqLineCheckBox;
  }

  /**
   * Constructs a panel with plot display options.
   */
  public PlotDisplayOptionPanel()
  {
    this(ConfigManager.DEFAULT_AMP_PHASE_SEL_IDX,
         ConfigManager.DEFAULT_LOG_AMP_FLAG,
         ConfigManager.DEFAULT_SHOW_POINTS_FLAG,
         ConfigManager.DEFAULT_COMBINE_AMP_PHASE_FLAG,
         ConfigManager.DEFAULT_SHOW_FREQ_LINE_FLAG);
  }

  /**
   * Constructs a panel with plot display options.
   * @param plotDisplayOptionPanel default plot display option panel
   */
  public PlotDisplayOptionPanel(
      PlotDisplayOptionPanel plotDisplayOptionPanel)
  {
    this(plotDisplayOptionPanel.getAmpPhaseComboBox().getSelectedIndex(),
         plotDisplayOptionPanel.getLogarithmicCheckBox().isSelected(),
         plotDisplayOptionPanel.getShowPointsCheckBox().isSelected(),
         plotDisplayOptionPanel.getCombineAmpPhaseCheckBox().isSelected(),
         plotDisplayOptionPanel.getShowFreqLineCheckBox().isSelected());
  }

  /**
   * Constructs a panel with plot display options.
   * @param ampPhaseSelIdx amp/phase selection index
   * @param logAmpFlag log amp flag
   * @param showPointsFlag show points flag
   * @param combineAmpPhaseFlag combine amp/phase flag
   * @param showFreqLineFlag show frequency line flag
   */
  public PlotDisplayOptionPanel(
      int ampPhaseSelIdx, boolean logAmpFlag, boolean showPointsFlag,
      boolean combineAmpPhaseFlag, boolean showFreqLineFlag)
  {
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

    //create amp/phase combo box:
    ampPhaseComboBox = new AmpPhaseComboBox();

    //create panel for label and combo box:
    ampPhaseComboBoxPanel = new JPanel();
    final JLabel ampPhaseComboBoxLabel = new JLabel("Display: ");
    ampPhaseComboBoxPanel.add(ampPhaseComboBoxLabel);      //add label
    ampPhaseComboBoxPanel.add(ampPhaseComboBox);           //add combo box
    final String ampPhaseComboBoxTipStr = "Select displayed graphs";
    ampPhaseComboBoxPanel.setToolTipText(ampPhaseComboBoxTipStr);
    ampPhaseComboBoxLabel.setToolTipText(ampPhaseComboBoxTipStr);
    ampPhaseComboBox.setToolTipText(ampPhaseComboBoxTipStr);

    //create "Logarithmic" check box:
    logarithmicCheckBox = new JCheckBox("Logarithmic Amplitude");
    logarithmicCheckBox.setToolTipText("Use logarithmic axis for amplitude");

    logarithmicPanel = new JPanel();
    logarithmicPanel.add(logarithmicCheckBox);

    //create "Show Datapoints" check box:
    showPointsCheckBox = new JCheckBox("Show Datapoints");
    showPointsCheckBox.setToolTipText("Highlight datapoints on graphs");

    showPointsPanel = new JPanel();
    showPointsPanel.add(showPointsCheckBox);

    //create "Combine Amp/Phase" check box:
    combineAmpPhaseCheckBox = new JCheckBox("Combine Amplitude/Phase");
    combineAmpPhaseCheckBox.setToolTipText(
                                "Show amplitude and phase on single graph");

    combineAmpPhasePanel = new JPanel();
    combineAmpPhasePanel.add(combineAmpPhaseCheckBox);

    //create "Show Freq Line" check box:
    showFreqLineCheckBox = new JCheckBox("Show Freq Line");
    showFreqLineCheckBox.setToolTipText(
                              "Show line marking frequency of sensitivity");

    showFreqLinePanel = new JPanel();
    showFreqLinePanel.add(showFreqLineCheckBox);

    //set default values
    ampPhaseComboBox.setSelectedIndex(ampPhaseSelIdx);
    logarithmicCheckBox.setSelected(logAmpFlag);
    showPointsCheckBox.setSelected(showPointsFlag);
    combineAmpPhaseCheckBox.setSelected(combineAmpPhaseFlag);
    showFreqLineCheckBox.setSelected(showFreqLineFlag);

    add(ampPhaseComboBoxPanel);  //add Amp/Phase combo-box panel
    add(showPointsPanel);        //add show points panel
    add(logarithmicPanel);       //add logarithmic panel
    add(combineAmpPhasePanel);   //add combine Amp/Phase panel
    add(showFreqLinePanel);      //add show freq line panel
  }

  /**
   * Constructs a panel with plot display options.
   * @param ampPhaseSelIdx amp/phase selection index
   * @param logAmpFlag log amp flag
   * @param showPointsFlag show points flag
   * @param combineAmpPhaseFlag combine amp/phase flag
   */
  public PlotDisplayOptionPanel(
      int ampPhaseSelIdx, boolean logAmpFlag, boolean showPointsFlag,
      boolean combineAmpPhaseFlag)
  {
    this(ampPhaseSelIdx,logAmpFlag,showPointsFlag,combineAmpPhaseFlag,
                                 ConfigManager.DEFAULT_SHOW_FREQ_LINE_FLAG);
  }

  /**
   * Add the components to the components table.
   * This should be done for the plot option panel on the entry panel.
   * @param configManager configuration manager.
   */
  protected void addToComponentsTable(ConfigManager configManager)
  {
    configManager.addToComponentsTable(
        ampPhaseComboBox, configManager.ampPhaseSelProp, true);
    configManager.addToComponentsTable(
        logarithmicCheckBox, configManager.logAmpFlagProp, true);
    configManager.addToComponentsTable(
        showPointsCheckBox, configManager.showPointsFlagProp, true);
    configManager.addToComponentsTable(
        combineAmpPhaseCheckBox, configManager.combineAmpPhaseFlagProp, true);
    configManager.addToComponentsTable(
        showFreqLineCheckBox, configManager.showFreqLineFlagProp, true);
  }

  /**
   * Sets the plot display options.
   * @param plotDisplayOptionPanel default plot display option panel.
   */
  public void setPlotDisplayOptions(
      PlotDisplayOptionPanel plotDisplayOptionPanel)
  {
    //set default values
    ampPhaseComboBox.setSelectedIndex(
        plotDisplayOptionPanel.getAmpPhaseComboBox().getSelectedIndex());
    logarithmicCheckBox.setSelected(
        plotDisplayOptionPanel.getLogarithmicCheckBox().isSelected());
    showPointsCheckBox.setSelected(
        plotDisplayOptionPanel.getShowPointsCheckBox().isSelected());
    combineAmpPhaseCheckBox.setSelected(
        plotDisplayOptionPanel.getCombineAmpPhaseCheckBox().isSelected());
    showFreqLineCheckBox.setSelected(
        plotDisplayOptionPanel.getShowFreqLineCheckBox().isSelected());
  }
}
