//RespChartPanel.java:  Generates and hosts the plotting of a response.
//
//   7/5/2002 -- [KF]  JFreeChart panel previously in RespPlotPanel,
//                     Plot phase using linear vertical axis,
//                     Use combined chart,
//                     Remove selected index from the data set,
//                     Add Combine Amp/Phase flag
//  7/16/2002 -- [ET]  Put in support for setting 'allowNegativesFlag'
//                     flag on 'VerticalLogarithmicAxis'.
//   3/5/2003 -- [KF]  Updates for JFreeChart 0.9.6.
//  5/12/2003 -- [ET]  Changed "com.jrefinery" to "org.jfree" for
//                     JFreeChart 0.9.8; added number-formatter override
//                     to linear vertical axis to fix value(s) being
//                     displayed with trailing ".0000000"; added 'get'
//                     methods for flags.
//  5/23/2003 -- [KF]  Modified to use dual axes for combined amp/phase plot
//                     and removed previous combined plot that used a
//                     call to 'setAllowNegativesFlag()'.
//  3/31/2005 -- [ET]  Added 'getLogAmpFlag()' method; modified to support
//                     showing graphs for all stages in a response; added
//                     'anyAmpValuesNotPositive()' method; setup tooltip
//                     generator and setup custom values for tooltip delays;
//                     added 2 significant digits to 'VNUMAXIS_FORMATTER'.
//  4/21/2005 -- [ET]  Added 'null' parameters to "new LegendItem()";
//                     modified "plotObj.setInsets()" to use
//                     'RectangleInsets'.
//  4/30/2007 -- [ET]  Changed 'XYDataset' reference from 'getSeriesName()'
//                     to 'getSeriesKey()'; changed 'StandardXYItemRenderer'
//                     usage to 'XYLineAndShapeRenderer'.
//  6/18/2007 -- [ET]  Modified to use 'ChartPanelWithCopy' class.
//   6/7/2010 -- [ET]  Added support for "Show Freq Line" checkbox.
//

package com.isti.jplotresp;

import java.util.Iterator;
import java.util.Collection;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import javax.swing.JPanel;
import javax.swing.JLabel;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;
import org.jfree.ui.Layer;
import org.jfree.data.xy.XYDataset;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartColor;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.SeriesRenderingOrder;
import org.jfree.chart.plot.DrawingSupplier;
import org.jfree.chart.plot.DefaultDrawingSupplier;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.event.PlotChangeEvent;
import org.jfree.chart.LegendItem;
import com.isti.jevalresp.AmpPhaseBlk;
import com.isti.jevalresp.OutputGenerator;

/**
 * Class RespChartPanel generates and hosts the plotting of a response.
 */
public class RespChartPanel extends JPanel
{
  protected int ampPhaseSelIdx;        //amp/phase display selection
  protected boolean showPointsFlag;    //true to show points on the plot
  protected boolean logAmpFlag;   //true to plot amp values logarithmically
  protected boolean combineAmpPhaseFlag;   //true for amp/phase on same plot
  protected boolean showFreqLineFlag;      //true to freq line on the plot
  protected final String title;   //title for plot
  protected final AmpPhaseBlk [] amplitudePhaseBlkArray;
  protected final OutputGenerator outputGeneratorObj;

  protected XYPlot plotObj;
  protected XYPlot ampPlotObj = null;
  protected ValueMarker senseFreqMarker = null;
  protected final NumberAxis xAxis;
  protected ChartPanel chartPanelObj = null;

  protected static final String AMP_LABEL_STR = "Amplitude";
  protected static final String PHASE_LABEL_STR = "Phase (degrees)";
  protected static final String FREQLN_LABEL_STR = "SensFreq";
              //shape object used by 'RespXYLineAndShapeRenderer':
  protected static final Shape RENDER_DEFAULT_SHAPE =
                                  new Rectangle2D.Double(-1.5,-1.5,3.0,3.0);
              //shape object used by 'RespXYLineAndShapeRenderer':
  protected static final Shape RENDER_LEGEND_SHAPE =
                                  new Rectangle2D.Double(-4.0,-4.0,8.0,8.0);

    //array of paint objects used to color each series in the chart:
  protected static final Paint [] seriesPaintDefaultsArray;
    //number formatter for linear vertical axis:
  protected static final NumberFormat VNUMAXIS_FORMATTER =
                                                 NumberFormat.getInstance();
  static
  {      //set number formatter to show no trailing digits for ".0":
    if(VNUMAXIS_FORMATTER instanceof DecimalFormat)
      ((DecimalFormat)VNUMAXIS_FORMATTER).applyPattern("#.###");
         //setup 'seriesPaintDefaultsArray' to contain an initial set
         // of colors followed by those in the JFreeChart array:
    final Paint [] paintDefsArr = new Paint []
          { Color.red, Color.blue, Color.green, Color.magenta, Color.orange,
                     Color.cyan, Color.darkGray, Color.pink, Color.yellow };
    if(paintDefsArr.length <
                       DefaultDrawingSupplier.DEFAULT_PAINT_SEQUENCE.length)
    {    //more items in JFreeChart array
      seriesPaintDefaultsArray =       //create array
            new Paint[DefaultDrawingSupplier.DEFAULT_PAINT_SEQUENCE.length];
      System.arraycopy(      //start with local set of colors
             paintDefsArr,0,seriesPaintDefaultsArray,0,paintDefsArr.length);
                             //finish with JFreeChart colors:
      System.arraycopy(DefaultDrawingSupplier.DEFAULT_PAINT_SEQUENCE,
           paintDefsArr.length,seriesPaintDefaultsArray,paintDefsArr.length,
                       DefaultDrawingSupplier.DEFAULT_PAINT_SEQUENCE.length-
                                                       paintDefsArr.length);
    }
    else      //not more items in JFreeChart array; use local array
      seriesPaintDefaultsArray = paintDefsArr;
  }

  /**
   * Generates and hosts the plotting of a response.
   * @param outGenObj output generator.
   * @param channelIdFName channel ID fname.
   * @param logSpacingFlag true to plot frequency values logarithmically.
   * @param plotDisplayOptionPanel plot display option panel.
   */
  public RespChartPanel(OutputGenerator outGenObj,
                        String channelIdFName,boolean logSpacingFlag,
                        PlotDisplayOptionPanel plotDisplayOptionPanel)
  {
    this.ampPhaseSelIdx =
        plotDisplayOptionPanel.getAmpPhaseComboBox().getSelectedIndex();
    this.showPointsFlag =
        plotDisplayOptionPanel.getShowPointsCheckBox().isSelected();
    this.logAmpFlag =
        plotDisplayOptionPanel.getLogarithmicCheckBox().isSelected();
    this.combineAmpPhaseFlag =
        plotDisplayOptionPanel.getCombineAmpPhaseCheckBox().isSelected();
    this.showFreqLineFlag =
        plotDisplayOptionPanel.getShowFreqLineCheckBox().isSelected();
    this.title = channelIdFName;
    outputGeneratorObj = outGenObj;
    setLayout(new BorderLayout());     //set layout manager

         //get array of amp/phase values (for all-stages response):
    if((amplitudePhaseBlkArray=outGenObj.getAmpPhaseArray()) == null)
      showErrorMessage(outGenObj.getErrorMessage());  //if error, show msg
         //create horizontal axis:
    final String freqStr = "Frequency (Hz)";
         //select logarithmic or linear horizontal axis and set
         // 'autoRangeIncludesZero' and 'autoRangeStickyZero'
         // flags to be false:
    xAxis = logSpacingFlag ?
            new LogarithmicAxis(freqStr) :
            new NumberAxis(freqStr);
    xAxis.setAutoRangeIncludesZero(false);
    xAxis.setAutoRangeStickyZero(false);

         //set initial amp/phase display selection and create plots:
    enterAmpPhaseSelection(ampPhaseSelIdx);
  }

  /**
   * Displays the given error message.
   * @param msgStr message string.
   */
  protected void showErrorMessage(String msgStr)
  {
    add(new JLabel(msgStr),BorderLayout.NORTH);
  }

  /**
   * Create the plot.
   * @param supplierObj drawing supplier to use.
   * @param datasetObj dataset to use.
   * @param ampPlotFlag true for amplitude plot; flag for phase plot.
   * @param domainAxis domain axis to use, or null for none..
   * @return A new plot object.
   */
  protected XYPlot createPlot(DrawingSupplier supplierObj,
            XYDataset datasetObj, boolean ampPlotFlag, ValueAxis domainAxis)
  {
         //create vertical axis:
    final NumberAxis yAxis = createVerticalNumberAxis(
                                                (logAmpFlag && ampPlotFlag),
                           (ampPlotFlag ? AMP_LABEL_STR : PHASE_LABEL_STR));
    final XYPlot plot = new XYPlot(datasetObj, domainAxis, yAxis,
                 new RespXYLineAndShapeRenderer(showPointsFlag,datasetObj));
    plot.setDrawingSupplier(supplierObj);
    plot.setSeriesRenderingOrder(SeriesRenderingOrder.REVERSE);
    return plot;
  }

  /**
   * Enters new amp/phase displayed selection.
   * @param ampPhaseSelIdx amp/phase display selection
   */
  public void enterAmpPhaseSelection(int ampPhaseSelIdx)
  {
    this.ampPhaseSelIdx = ampPhaseSelIdx;
    createPlots();  //create the plots
  }

  /**
   * Returns the amp/phase displayed selection.
   * @return The amp/phase displayed selection.
   */
  public int getAmpPhaseSelection()
  {
    return ampPhaseSelIdx;
  }

  /**
   * Create a vertical number axis for the specified series.
   * @param logAxisFlag true for logarithmic axis; false for linear axis.
   * @param labelStr label string for axis.
   * @return A new vertical number axis object.
   */
  protected NumberAxis createVerticalNumberAxis(boolean logAxisFlag,
                                                            String labelStr)
  {
    final NumberAxis yAxis;
    if(logAxisFlag)
    {    //flag set; create log axis
      yAxis = new LogarithmicAxis(labelStr);
      yAxis.setAutoRangeMinimumSize(1e-29);    //allow range of small values
      yAxis.setLowerMargin(0.1);  //make lower margin larger than default
                   //setup to show "1e"-style labels:
      ((LogarithmicAxis)yAxis).setLog10TickLabelsFlag(false);
      ((LogarithmicAxis)yAxis).setExpTickLabelsFlag(true);
                   //setup to not auto range to next "10^n" value:
      ((LogarithmicAxis)yAxis).setAutoRangeNextLogFlag(false);
    }
    else
    {    //flag not set; create linear axis
      yAxis = new NumberAxis(labelStr);
              //use formatter to avoid ".0000000" displays:
      yAxis.setNumberFormatOverride(VNUMAXIS_FORMATTER);
    }
    yAxis.setAutoRangeIncludesZero(false);
    yAxis.setAutoRangeStickyZero(false);
    return yAxis;
  }

  /**
   * Creates the plots.
   */
  protected void createPlots()
  {
    int numberOfGraphs = 1;
         //create dataset for amplitude values:
    final AmpPhaseDataset ampDatasetObj;
    if(!AmpPhaseComboBox.isPhaseSelIdx(ampPhaseSelIdx))
    {    //not showing only phase values
                   //use constructor for showing all stages
                   // or constructor for showing final response only:
      ampDatasetObj = AmpPhaseComboBox.isShowStagesSelIdx(ampPhaseSelIdx) ?
                                                        new AmpPhaseDataset(
                     outputGeneratorObj.getAllStagesAmpPhaseArrays(),true) :
                  new AmpPhaseDataset(outputGeneratorObj.getCalcFreqArray(),
                                               amplitudePhaseBlkArray,true);
                   //setup number of graphs per plot:
      numberOfGraphs = ampDatasetObj.getSeriesCount();
    }
    else      //showing only phase values
      ampDatasetObj = null;       //amplitude dataset not needed
         //create dataset for phase values:
    final AmpPhaseDataset phaseDatasetObj;
    if(!AmpPhaseComboBox.isAmpSelIdx(ampPhaseSelIdx))
    {    //not showing only amplitude values
                   //use constructor for showing all stages
                   // or constructor for showing final response only:
      phaseDatasetObj = AmpPhaseComboBox.isShowStagesSelIdx(ampPhaseSelIdx) ?
                                                        new AmpPhaseDataset(
                    outputGeneratorObj.getAllStagesAmpPhaseArrays(),false) :
                  new AmpPhaseDataset(outputGeneratorObj.getCalcFreqArray(),
                                              amplitudePhaseBlkArray,false);
                   //setup number of graphs per plot:
      numberOfGraphs = phaseDatasetObj.getSeriesCount();
    }
    else      //showing only amplitude values
      phaseDatasetObj = null;     //phase dataset not needed

    final DrawingSupplier supplierObj;
    Paint [] seriesPaintArr;
    if(AmpPhaseComboBox.isShowStagesSelIdx(ampPhaseSelIdx))
    {    //showing all stages of response
      try
      {
        if(AmpPhaseComboBox.isAmpPhaseSelIdx(ampPhaseSelIdx) &&
                                                        combineAmpPhaseFlag)
        {     //showing amp & phase in a combined graph
                   //setup to have graphs use different colors for all lines
                        //if more graphs than colors then limit:
          if(numberOfGraphs >= seriesPaintDefaultsArray.length)
            numberOfGraphs = seriesPaintDefaultsArray.length - 1;
                   //create paint array with same length as default:
          seriesPaintArr = new Paint[seriesPaintDefaultsArray.length];
                    //start with first color from defaults array (red):
          seriesPaintArr[0] = seriesPaintDefaultsArray[0];
          int i;    //put in set of amp colors for stages:
          for(i=1; i<numberOfGraphs; ++i)
            seriesPaintArr[i] = seriesPaintDefaultsArray[i+1];
                    //put in second color from defaults array (blue):
          seriesPaintArr[i] = seriesPaintDefaultsArray[1];
                    //put in rest of colors (including stages phase colors):
          while(++i < seriesPaintArr.length)
            seriesPaintArr[i] = seriesPaintDefaultsArray[i];
        }
        else
        {     //not showing amp & phase in a combined graph
                   //setup to have graphs use same colors for stage lines
                        //if too many graphs then limit:
          if(numberOfGraphs*2 > seriesPaintDefaultsArray.length)
            numberOfGraphs = seriesPaintDefaultsArray.length / 2;
                                  //create paint array with room enough
          seriesPaintArr =        // to repeat colors for stages (phase):
                new Paint[seriesPaintDefaultsArray.length+numberOfGraphs-1];
                    //start with first color from defaults array (red):
          seriesPaintArr[0] = seriesPaintDefaultsArray[0];
          int i;    //put in set of amp colors for stages:
          for(i=1; i<numberOfGraphs; ++i)
            seriesPaintArr[i] = seriesPaintDefaultsArray[i+1];
                    //put in second color from defaults array (blue):
          seriesPaintArr[i] = seriesPaintDefaultsArray[1];
                    //put in rest of colors (incl repeat stages phase clrs):
          while(++i < seriesPaintArr.length)
            seriesPaintArr[i] = seriesPaintDefaultsArray[i-numberOfGraphs+1];
        }
      }
      catch(Exception ex)
      {       //some kind of exception error; show message
        System.err.println("Error modifying graph paint order:");
        ex.printStackTrace();
                   //use default paint array:
        seriesPaintArr = seriesPaintDefaultsArray;
      }
    }
    else      //not showing all stages of response; use default paint array
      seriesPaintArr = seriesPaintDefaultsArray;
              //create drawing supplier object for graphs:
    supplierObj = new DefaultDrawingSupplier(seriesPaintArr,
                      DefaultDrawingSupplier.DEFAULT_OUTLINE_PAINT_SEQUENCE,
                             DefaultDrawingSupplier.DEFAULT_STROKE_SEQUENCE,
                     DefaultDrawingSupplier.DEFAULT_OUTLINE_STROKE_SEQUENCE,
                             DefaultDrawingSupplier.DEFAULT_SHAPE_SEQUENCE);

    xAxis.setAutoRange(true);     //make sure X-axis is zoomed out
    if(AmpPhaseComboBox.isAmpPhaseSelIdx(ampPhaseSelIdx))
    {    //both amplitude and phase plots shown
      if(!combineAmpPhaseFlag)
      {  //showing separate graphs
        plotObj = new CombinedDomainXYPlot(xAxis);
        plotObj.setDrawingSupplier(supplierObj);
        plotObj.setSeriesRenderingOrder(SeriesRenderingOrder.REVERSE);
        ((CombinedDomainXYPlot)plotObj).add(ampPlotObj=
                           createPlot(supplierObj,ampDatasetObj,true,null));
        ((CombinedDomainXYPlot)plotObj).add(
                        createPlot(supplierObj,phaseDatasetObj,false,null));
      }
      else
      {  //showing combined graph
        plotObj = ampPlotObj =
                           createPlot(supplierObj,ampDatasetObj,true,xAxis);
        plotObj.setRangeAxis(1,
                           createVerticalNumberAxis(false,PHASE_LABEL_STR));
              //associate dataset #1 with range-axis #1 (need to do this
              // before entering dataset to prevent possible momentary
              // plotting of non-positive values on amplitude's log axis):
        plotObj.mapDatasetToRangeAxis(1,1);
        plotObj.setDataset(1,phaseDatasetObj);
        plotObj.setRenderer(1,
            new RespXYLineAndShapeRenderer(showPointsFlag,phaseDatasetObj));
      }
    }
    else
    {    //only amplitude or only phase plot shown
      final boolean ampPlotFlag =
                               AmpPhaseComboBox.isAmpSelIdx(ampPhaseSelIdx);
      if(!ampPlotFlag)
      {  //phase-only plot
        if(AmpPhaseComboBox.isShowStagesSelIdx(ampPhaseSelIdx))
        {     //showing all stages of response
                   //skip ahead to "phase" colors in paint sequence:
          for(int i=0; i<numberOfGraphs; ++i)
            supplierObj.getNextPaint();
        }
        else       //not showing all stages of response
          supplierObj.getNextPaint();         //skip to second paint color
      }
      plotObj = createPlot(supplierObj,
                            (ampPlotFlag ? ampDatasetObj : phaseDatasetObj),
                                                         ampPlotFlag,xAxis);
      ampPlotObj = ampPlotFlag ? plotObj : null;
    }
         //add margin around plots (need room for right-side labels):
    plotObj.setInsets(new RectangleInsets(0.0,5.0,0.0,25.0));
    if(showFreqLineFlag)     //if frequency line enabled then
      showFreqLine();        //show frequency line

         //create chart with X-Y plot object:
    final JFreeChart chartObj = new JFreeChart(
                          title,JFreeChart.DEFAULT_TITLE_FONT,plotObj,true);
         //make chart background same as panel:
    chartObj.setBackgroundPaint(getBackground());

    if (chartPanelObj != null)           //if chart panel previously created
      chartPanelObj.setChart(chartObj);  // then enter new chart object
    else
    {    //chart panel not previously created
              //put chart object into 'ChartPanel':
      chartPanelObj = new ChartPanelWithCopy(chartObj);
              //setup delay values for tooltips:
      chartPanelObj.setInitialDelay(0);          //no delay before show
      chartPanelObj.setReshowDelay(0);           //no delay before reshow
      chartPanelObj.setDismissDelay(30000);      //show for up to 30 seconds
              //add chart panel to this panel:
      add(chartPanelObj, BorderLayout.CENTER);
    }

         //notify all registered listeners that plot has been modified:
    plotObj.notifyListeners(new PlotChangeEvent(plotObj));
  }

  /**
   * Selects if the amp and phase plots should be combined on the chart.
   * @param b true to combine amp/phase on the same plot.
   */
  public void setCombineAmpPhaseFlag(boolean b)
  {
    if(b != combineAmpPhaseFlag)
    {  //flag value is changing
      combineAmpPhaseFlag = b;         //set the flag value
              //set amp/phase display selection and create plots:
      enterAmpPhaseSelection(ampPhaseSelIdx);
    }
  }

  /**
   * Returns a flag indicating whether or not the amp and phase plots
   * should be combined on the chart.
   * @return true if amp/phase combined on the same plot.
   */
  public boolean getCombineAmpPhaseFlag()
  {
    return combineAmpPhaseFlag;
  }

  /**
   * Selects if data-points are shown on the chart.
   * @param b true to show data-points on the plot.
   */
  public void setShowPointsFlag(boolean b)
  {
    showPointsFlag = b;
    setShowPointsFlag(plotObj, b);
         //notify all registered listeners that plot has been modified:
    plotObj.notifyListeners(new PlotChangeEvent(plotObj));
  }

  /**
   * Selects if data-points are shown on the chart.
   * @param plot the plot.
   * @param b true to show data-points on the plot.
   */
  protected void setShowPointsFlag(XYPlot plot, boolean b)
  {
    if (plot instanceof CombinedDomainXYPlot)
    {
      //for each subplot
      Iterator it = ((CombinedDomainXYPlot)plot).getSubplots().iterator();
      while (it.hasNext())
      {
        //set the plot shapes according to the show points flag
        setShowPointsFlag((XYPlot)it.next(), b);
      }
    }
    else if (plot instanceof XYPlot)
    {
      //if renderer is standard
      if (plot.getRenderer() instanceof XYLineAndShapeRenderer)
      {
        ((XYLineAndShapeRenderer)plot.getRenderer()).
                                                    setBaseShapesVisible(b);
      }
      //if secondary renderer is standard
      if (plot.getRenderer(1) instanceof XYLineAndShapeRenderer)
      {
        ((XYLineAndShapeRenderer)plot.getRenderer(1)).
                                                    setBaseShapesVisible(b);
      }
    }
  }

  /**
   * Returns a flag indicating whether or not data-points are shown on
   * the chart.
   * @return true if data-points are shown on the chart.
   */
  public boolean getShowPointsFlag()
  {
    return showPointsFlag;
  }

  /**
   * Selects if the amplitude should be shown logarithmically.
   * @param b true to plot amp values logarithmically.
   */
  public void setLogAmpFlag(boolean b)
  {
    if(b != logAmpFlag)
    {  //flag value is changing
      logAmpFlag = b;        //set flag value
      createPlots();         //recreate the plots
    }
  }

  /**
   * Returns a flag indicating whether or not the amplitude should be
   * shown logarithmically.
   * @return true if amp values are shown logarithmically.
   */
  public boolean getLogAmpFlag()
  {
    return logAmpFlag;
  }

  /**
   * Selects if sensitivity-frequency line is shown on the chart.
   * @param b true to show sensitivity-frequency line on the plot.
   */
  public void setShowFreqLineFlag(boolean b)
  {
    if(b != showFreqLineFlag)
    {  //flag value is changing
      showFreqLineFlag = b;       //set flag value
      if(showFreqLineFlag)        //if flag set then
        showFreqLine();           //show frequency line
      else
      {  //clearing frequency line
        if(!clearFreqLine())           //if clear failed then
          createPlots();               //recreate the plots
      }
    }
  }

  /**
   * Displays the frequency-sensitivity line.
   */
  protected void showFreqLine()
  {
    if(ampPlotObj != null)
    {  //amplitude plot was created
      final Float senseFreqObj;
      if((senseFreqObj=outputGeneratorObj.getRespSensitFreq()) != null)
      {  //sensitivity frequency fetched OK
        senseFreqMarker =         //create marker with value
                       new ValueMarker((double)(senseFreqObj.floatValue()));
                                  //set color for marker line:
        senseFreqMarker.setPaint(ChartColor.VERY_DARK_GREEN);
        senseFreqMarker.setLabel(FREQLN_LABEL_STR);   //set displayed label
                                                      //set label position:
        senseFreqMarker.setLabelAnchor(RectangleAnchor.BOTTOM);
        senseFreqMarker.setLabelTextAnchor(TextAnchor.BOTTOM_CENTER);
                                                      //add marker to plot:
        ampPlotObj.addDomainMarker(senseFreqMarker,Layer.FOREGROUND);
      }
    }
  }

  /**
   * Clears the frequency-sensitivity line.
   * @return true if the frequency-sensitivity line was cleared;
   * false if not.
   */
  protected boolean clearFreqLine()
  {
    final Collection mkrsList;
    if(ampPlotObj != null && senseFreqMarker != null &&
         (mkrsList=ampPlotObj.getDomainMarkers(Layer.FOREGROUND)) != null &&
                 mkrsList.size() == 1 && mkrsList.contains(senseFreqMarker))
    {  //amplitude plot was created and list with marker fetched OK
      senseFreqMarker = null;               //clear handle for marker
      ampPlotObj.clearDomainMarkers();      //clear marker from plot
      return true;
    }
    return false;
  }

  /**
   * Returns a flag indicating whether or not sensitivity-frequency line
   * is shown on the chart.
   * @return true if sensitivity-frequency line is shown on the chart.
   */
  public boolean getShowFreqLineFlag()
  {
    return showFreqLineFlag;
  }

  /**
   * Determines if any amplitude values are less than or equal to zero.
   * @return true if any amplitude values are less than or equal to zero.
   */
  public boolean anyAmpValuesNotPositive()
  {
    return AmpPhaseComboBox.isShowStagesSelIdx(ampPhaseSelIdx) ?
                          outputGeneratorObj.allStagesAnyAmpsNotPositive() :
                                    outputGeneratorObj.anyAmpsNotPositive();
  }

  /**
   * Creates a print job for the chart.
   */
  public void createChartPrintJob()
  {
    chartPanelObj.createChartPrintJob();
  }

  /**
   * Opens a file chooser and gives the user an opportunity to save the chart
   * in PNG format.
   */
  public void doSaveAs()
  {
    try
    {
      chartPanelObj.doSaveAs();
    }
    catch (Exception ex)
    {
      System.out.println(
          "Error saving the chart:\n" +
          ex);
    }
  }

  /**
   * Class RespXYLineAndShapeRenderer extends XYLineAndShapeRenderer to
   * support a 'showPointsFlag' parameter, define a smaller shape
   * that is always returned via the 'getSeriesShape()' method, and
   * show "square" shapes on the legend items.
   */
  protected static class RespXYLineAndShapeRenderer
                                              extends XYLineAndShapeRenderer
  {
    /**
     * Constructs a new renderer.
     * @param showPointsFlag true to show points on the plot.
     * @param datasetObj if an 'AmpPhaseDataset' then its tooltip
     * generator object is used.
     */
    public RespXYLineAndShapeRenderer(boolean showPointsFlag,
                                                       XYDataset datasetObj)
    {
      setBaseShapesVisible(showPointsFlag);    //set initial show-points flg
      if(datasetObj instanceof AmpPhaseDataset)
      {  //given dataset is an 'AmpPhaseDataset'
        setToolTipGenerator(      //enter tooltip generator from dataset
            ((AmpPhaseDataset)datasetObj).getAmpPhaseTooltipGeneratorObj());
      }
    }

    /**
     * Returns a shape used to represent the items in a series.
     * This override of the method always returns the default shape.
     * @param series  the series (zero-based index).
     * @return The shape object.
     */
    public Shape getSeriesShape(int series)
    {
      return RENDER_DEFAULT_SHAPE;
    }

    /**
     * Returns a legend item for a series.  This override of the method
     * uses 'RENDER_LEGEND_SHAPE' to create the legend item.
     * @param datasetIndex  dataset index (zero-based).
     * @param series series index (zero-based).
     * @return A new legend item for the series (or null if error).
     */
    public LegendItem getLegendItem(int datasetIndex, int series)
    {
      LegendItem result = null;

      final XYPlot xyplot = getPlot();
      if (xyplot != null)
      {
        final XYDataset dataset = xyplot.getDataset(datasetIndex);
        if (dataset != null)
        {
          if (getItemVisible(series, 0))
          {
            final Comparable compObj;
            final String label =
                          ((compObj=dataset.getSeriesKey(series)) != null) ?
                                                    compObj.toString() : "";
            final Shape shape = RENDER_LEGEND_SHAPE;
            final Paint paint = getSeriesPaint(series);
            final Paint outlinePaint = getSeriesOutlinePaint(series);
            final Stroke outlineStroke = getSeriesOutlineStroke(series);
            result = new LegendItem(label,label,null,null,shape,paint,
                                                outlineStroke,outlinePaint);
          }
        }
      }
      return result;
    }
  }
}
