//RespPlotPanel.java:  Generates and hosts the plotting of a response.
//
//   5/16/2002 -- [ET]
//   6/11/2002 -- [KF]  Updates for JFreeChart 0.9.0
//    7/5/2002 -- [KF]  Moved JFreeChart panel into a separate class
//                      (RespChartPanel), added "Combine Amp/Phase"
//                      checkbox and default to separate Amplitude/Phase
//                      plots.
//    8/6/2002 -- [ET]  Improved response information displayed below plot
//                      (same as output-file header information).
//    3/5/2003 -- [KF]  Added Print and Save buttons.
//   5/12/2003 -- [ET]  Changed ActionListeners to ChangeListeners on
//                      checkbox buttons to get events fired when
//                      'setSelected()' method called; modified listeners
//                      to check if item changed; added tooltips to
//                      buttons.
//   4/11/2005 -- [ET]  Modified to make change-graph controls revert if
//                      error generating graph; modified to not create
//                      copy of 'plotDisplayOptionPanel' object.
//    6/7/2010 -- [ET]  Added support for "Show Freq Line" checkbox.
//

package com.isti.jplotresp;

import java.util.Date;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import edu.iris.Fissures.IfNetwork.ChannelId;
import edu.iris.Fissures.model.ISOTime;
import com.isti.jevalresp.RespUtils;
import com.isti.jevalresp.OutputGenerator;

/**
 * Class RespPlotPanel generates and hosts the plotting of a response.
 */
public class RespPlotPanel extends JPanel
{
  protected final JPanel contentPanelObj = new JPanel();
  protected final JTabbedPane tabbedPaneObj;
  protected final String channelIdFName;
  protected final RespChartPanel chartPanel;
  protected final PlotDisplayOptionPanel plotDisplayOptionPanel;

  /**
   * Creates a panel object that generates and hosts the plotting of a
   * response.
   * @param tabbedPaneObj tabbed-pane object
   * @param outGenObj the 'OutputGenerator' object to use.
   * @param channelIdObj the channed ID name
   * @param respEndDateObj response end date object
   * @param channelIdFName the output filename to use.
   * @param logSpacingFlag true to plot frequency values logarithmically.
   * @param plotDispOptPanel default plot display option panel.
   */
  public RespPlotPanel(JTabbedPane tabbedPaneObj, OutputGenerator outGenObj,
                       ChannelId channelIdObj, Date respEndDateObj,
                       String channelIdFName, boolean logSpacingFlag,
                       PlotDisplayOptionPanel plotDispOptPanel)
  {
    this.tabbedPaneObj = tabbedPaneObj;     //save tabbed-pane object
    this.channelIdFName = channelIdFName;   //save channed ID name
    this.plotDisplayOptionPanel = plotDispOptPanel;

    //create text area of information about response:
    final JTextArea textAreaObj = new JTextArea(
        RespUtils.channelIdToHdrString(
        channelIdObj,respEndDateObj,",  ","\n") + "\n" +
        ((outGenObj!=null)?outGenObj.getOutputHeaderString(",  ","\n"):""));
    textAreaObj.setEditable(false);         //set text not editable
    //setup font for text:
    textAreaObj.setFont((new JLabel()).getFont().deriveFont(Font.BOLD,13));
    //make background same as host panel:
    textAreaObj.setBackground(getBackground());

    //put text area in panel to center it:
    final JPanel textAreaPanel = new JPanel();
    //set empty border around panel for margin:
    textAreaPanel.setBorder(BorderFactory.createEmptyBorder(1,5,10,1));
    textAreaPanel.add(textAreaObj);         //put text area in panel


    //create "Print" button:
    final JButton printButton = new JButton("Print");
    //setup "Print" button action
    printButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evtObj)
      {
        chartPanel.createChartPrintJob();
      }
    });
    printButton.setToolTipText("Send graphs to printer");

    //create "Save" button:
    final JButton saveButton = new JButton("Save");
    //setup "Save" button action
    saveButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evtObj)
      {
        chartPanel.doSaveAs();
      }
    });
    saveButton.setToolTipText("Save graphs to PNG file");

    //create "Close" button:
    final JButton closeButton = new JButton("Close");
    closeButton.addActionListener(new ActionListener()
    {
      //setup "Close" button action
      public void actionPerformed(ActionEvent evtObj)
      {
        //remove tab from display:
        if(RespPlotPanel.this.tabbedPaneObj != null)
        {
          RespPlotPanel.this.tabbedPaneObj.remove(RespPlotPanel.this);
        }
      }
    });
    closeButton.setToolTipText("Close this panel");

    final JPanel buttonPanel = new JPanel();     //create panel for buttons
    buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
    buttonPanel.add(Box.createGlue());
    buttonPanel.add(printButton);                //add print button to panel
    buttonPanel.add(Box.createGlue());
    buttonPanel.add(saveButton);                 //add save button to panel
    buttonPanel.add(Box.createGlue());
    buttonPanel.add(closeButton);                //add close button to panel
    buttonPanel.add(Box.createGlue());

    plotDisplayOptionPanel.getAmpPhaseComboBox().addActionListener(
                                                        new ActionListener()
        {
          boolean inhibitChangeFlag = false;

          //setup combo-box selection action
          public void actionPerformed(ActionEvent evtObj)
          {
            if(inhibitChangeFlag)
            { //state-change action inhibited
              inhibitChangeFlag = false;    //make sure flag is cleared
              return;                       //exit method
            }
            final AmpPhaseComboBox ampPhaseComboBoxObj =
                               plotDisplayOptionPanel.getAmpPhaseComboBox();
            if(ampPhaseComboBoxObj.getSelectedIndex() !=
                                          chartPanel.getAmpPhaseSelection())
            {      //amp/phase combo-box selection is changing
              //change selection of amp/phase displayed
              SwingUtilities.invokeLater(new Runnable()
                  {
                    //run this later so combo box responds quickly
                    public void run()
                    {
                      int selVal = 0;
                      try
                      {
                        selVal = chartPanel.getAmpPhaseSelection();
                        chartPanel.enterAmpPhaseSelection(
                                    ampPhaseComboBoxObj.getSelectedIndex());
                      }
                      catch(Exception ex)
                      {      //error changing graph; show error message
                        System.err.println("Error changing graph:");
                        ex.printStackTrace();
                             //revert to previous type of graph:
                                  //inhibit state-change action:
                        inhibitChangeFlag = true;
                                  //revert combo-box selection:
                        ampPhaseComboBoxObj.setSelectedIndex(selVal);
                                  //restore state-change action:
                        inhibitChangeFlag = false;
                                  //revert graph:
                        chartPanel.enterAmpPhaseSelection(selVal);
                      }
                    }
                  });
            }
          }
        });

    plotDisplayOptionPanel.getShowPointsCheckBox().addChangeListener(
                                                        new ChangeListener()
        {
          boolean inhibitChangeFlag = false;

          //setup check box action
          public void stateChanged(ChangeEvent evtObj)
          {
            if(inhibitChangeFlag)
            { //state-change action inhibited
              inhibitChangeFlag = false;    //make sure flag is cleared
              return;                       //exit method
            }
            final JCheckBox showPointsCheckBox =
                             plotDisplayOptionPanel.getShowPointsCheckBox();
            if(showPointsCheckBox.isSelected() !=
                                             chartPanel.getShowPointsFlag())
            {     //show-points state is changing
              //change item renderer:
              SwingUtilities.invokeLater(new Runnable()
                  {
                    //run this later so check box responds quickly
                    public void run()
                    {
                      boolean chkBoxFlag = false;
                      try
                      {
                        chartPanel.setShowPointsFlag(
                                           showPointsCheckBox.isSelected());
                      }
                      catch(Exception ex)
                      {      //error changing graph; show error message
                        System.err.println("Error changing graph:");
                        ex.printStackTrace();
                             //revert to previous type of graph:
                        chkBoxFlag = !chkBoxFlag;
                                  //inhibit state-change action:
                        inhibitChangeFlag = true;
                                  //revert state of check-box:
                        showPointsCheckBox.setSelected(chkBoxFlag);
                                  //restore state-change action:
                        inhibitChangeFlag = false;
                                  //revert graph:
                        chartPanel.setLogAmpFlag(chkBoxFlag);
                      }
                    }
                  });
            }
          }
        });

    plotDisplayOptionPanel.getCombineAmpPhaseCheckBox().addChangeListener(
                                                        new ChangeListener()
        {
          boolean inhibitChangeFlag = false;

          //setup check box action
          public void stateChanged(ChangeEvent evtObj)
          {
            if(inhibitChangeFlag)
            { //state-change action inhibited
              inhibitChangeFlag = false;    //make sure flag is cleared
              return;                       //exit method
            }
            final JCheckBox combineAmpPhaseCheckBox =
                        plotDisplayOptionPanel.getCombineAmpPhaseCheckBox();

            if(combineAmpPhaseCheckBox.isSelected() !=
                                        chartPanel.getCombineAmpPhaseFlag())
            {      //combine-amp/phase state is changing
                        //modify chart:
              SwingUtilities.invokeLater(new Runnable()
                  {
                    //run this later so check box responds quickly
                    public void run()
                    {
                      boolean chkBoxFlag = false;
                      try
                      {
                        chartPanel.setCombineAmpPhaseFlag(
                                      combineAmpPhaseCheckBox.isSelected());
                      }
                      catch(Exception ex)
                      {      //error changing graph; show error message
                        System.err.println("Error changing graph:");
                        ex.printStackTrace();
                             //revert to previous type of graph:
                        chkBoxFlag = !chkBoxFlag;
                                  //inhibit state-change action:
                        inhibitChangeFlag = true;
                                  //revert state of check-box:
                        combineAmpPhaseCheckBox.setSelected(chkBoxFlag);
                                  //restore state-change action:
                        inhibitChangeFlag = false;
                                  //revert graph:
                        chartPanel.setLogAmpFlag(chkBoxFlag);
                      }
                    }
                  });
            }
          }
        });

    plotDisplayOptionPanel.getLogarithmicCheckBox().addChangeListener(
                                                        new ChangeListener()
        {
          boolean inhibitChangeFlag = false;

          //setup check box action
          public void stateChanged(ChangeEvent evtObj)
          {
            if(inhibitChangeFlag)
            { //state-change action inhibited
              inhibitChangeFlag = false;    //make sure flag is cleared
              return;                       //exit method
            }
            final JCheckBox logarithmicCheckBox =
                            plotDisplayOptionPanel.getLogarithmicCheckBox();
            if(logarithmicCheckBox.isSelected() != chartPanel.getLogAmpFlag())
            {      //show-log state is changing
                        //modify chart:
              SwingUtilities.invokeLater(new Runnable()
                  {
                    //run this later so check box responds quickly
                    public void run()
                    {
                      boolean chkBoxFlag = false;
                      try
                      {
                        if((chkBoxFlag=logarithmicCheckBox.isSelected()) &&
                                       chartPanel.anyAmpValuesNotPositive())
                        {    //switching to log graph and amp values <= 0
                                  //show error message
                          System.err.println(
                                         RespPlotPanel.this.channelIdFName +
                                             ":  Amplitude data contains " +
                                          "non-positive value(s); cannot " +
                                              "switch to logarithmic plot");
                          //revert chart setting below
                        }
                        else
                        {
                          chartPanel.setLogAmpFlag(chkBoxFlag);
                          return;
                        }
                      }

                      catch(Exception ex)
                      {      //error changing graph; show error message
                        System.err.println(
                                         RespPlotPanel.this.channelIdFName +
                                                ":  Error changing graph:");
                        ex.printStackTrace();
                        //revert chart setting below
                      }
                             //revert to previous type of graph:
                      chkBoxFlag = !chkBoxFlag;
                             //inhibit state-change action:
                      inhibitChangeFlag = true;
                             //revert state of check-box:
                      logarithmicCheckBox.setSelected(chkBoxFlag);
                             //restore state-change action:
                      inhibitChangeFlag = false;
                             //revert graph:
                      chartPanel.setLogAmpFlag(chkBoxFlag);
                    }
                  });
            }
          }
        });

    plotDisplayOptionPanel.getShowFreqLineCheckBox().addChangeListener(
                                                        new ChangeListener()
        {
          boolean inhibitChangeFlag = false;

          //setup check box action
          public void stateChanged(ChangeEvent evtObj)
          {
            if(inhibitChangeFlag)
            { //state-change action inhibited
              inhibitChangeFlag = false;    //make sure flag is cleared
              return;                       //exit method
            }
            final JCheckBox showFreqLineCheckBox =
                           plotDisplayOptionPanel.getShowFreqLineCheckBox();
            if(showFreqLineCheckBox.isSelected() !=
                                           chartPanel.getShowFreqLineFlag())
            {     //show-freq-line state is changing
              //change item renderer:
              SwingUtilities.invokeLater(new Runnable()
                  {
                    //run this later so check box responds quickly
                    public void run()
                    {
                      boolean chkBoxFlag = false;
                      try
                      {
                        chartPanel.setShowFreqLineFlag(
                                         showFreqLineCheckBox.isSelected());
                      }
                      catch(Exception ex)
                      {      //error changing graph; show error message
                        System.err.println("Error changing graph:");
                        ex.printStackTrace();
                             //revert to previous type of graph:
                        chkBoxFlag = !chkBoxFlag;
                                  //inhibit state-change action:
                        inhibitChangeFlag = true;
                                  //revert state of check-box:
                        showFreqLineCheckBox.setSelected(chkBoxFlag);
                                  //restore state-change action:
                        inhibitChangeFlag = false;
                                  //revert graph:
                        chartPanel.setLogAmpFlag(chkBoxFlag);
                      }
                    }
                  });
            }
          }
        });

    //create lower vertical-box panel:
    final Box lowerBoxObj = Box.createVerticalBox();
    lowerBoxObj.add(textAreaPanel);              //add informational text
    lowerBoxObj.add(plotDisplayOptionPanel);   //add display option pnanel
    lowerBoxObj.add(buttonPanel);              //add button panel

    //set content panel layout manager (gap between chart & button):
    contentPanelObj.setLayout(new BorderLayout(0,10));
    //setup empty border for margin:
    contentPanelObj.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    //add chart to panel:
    chartPanel = new RespChartPanel(outGenObj, channelIdFName, logSpacingFlag,
                                    plotDisplayOptionPanel);

    //set up defaults from chart panel
    plotDisplayOptionPanel.getShowPointsCheckBox().setSelected(
        chartPanel.getShowPointsFlag());
    plotDisplayOptionPanel.getLogarithmicCheckBox().setSelected(
        chartPanel.getLogAmpFlag());
    plotDisplayOptionPanel.getCombineAmpPhaseCheckBox().setSelected(
        chartPanel.getCombineAmpPhaseFlag());
    plotDisplayOptionPanel.getShowFreqLineCheckBox().setSelected(
        chartPanel.getShowFreqLineFlag());

    contentPanelObj.add(chartPanel,BorderLayout.CENTER);
    //add panel below chart:
    contentPanelObj.add(lowerBoxObj,BorderLayout.SOUTH);

    setLayout(new BorderLayout());     //set layout manager
    //put content in scroll pane and add:
    add(new JScrollPane(contentPanelObj),BorderLayout.CENTER);
  }

  /**
   * Sets the plot display options.
   * @param plotDispOptPanel default plot display option panel.
   */
  public void setPlotDisplayOptions(
      PlotDisplayOptionPanel plotDispOptPanel)
  {
    //set the plot options to the defaults
    plotDisplayOptionPanel.setPlotDisplayOptions(plotDispOptPanel);
  }

  /**
   * Converts the given channel ID object to an information string.
   * @param channelIdObj the channed ID name
   * @return information string
   */
  public static String channelIdToInfoString(ChannelId channelIdObj)
  {
    String retStr;
    if(channelIdObj != null)
    {
      retStr = "Station: " + channelIdObj.station_code + ",  Channel: " +
               channelIdObj.channel_code;
      if(channelIdObj.network_id != null)
        retStr += ",  Network: " + channelIdObj.network_id.network_code;
      if(channelIdObj.site_code != null &&
         channelIdObj.site_code.trim().length() > 0)
      {
        retStr += ",  Location: " + channelIdObj.site_code;
      }
      if(channelIdObj.begin_time != null)
      {
        String dateStr;
        try
        {
          final Date dateObj =   //convert FISSURES time string to Date
              (new ISOTime(channelIdObj.begin_time.date_time)).getDate();
          //convert Date to 'evalresp' time string:
          dateStr = RespUtils.respDateFormatter.format(dateObj);
        }
        catch(Exception ex)
        {
          //error converting dates
          dateStr = "????";       //indicate error
        }
        retStr += ",  BeginTime: " + dateStr;
      }
    }
    else
      retStr = "";
    return retStr;
  }
}
