//RespProcessingThread.java:  Creates a new thread to process parameters
//
//  5/12/2002 -- [KF]
//  5/15/2002 -- [ET]
//  7/10/2002 -- [ET]  Added dummy version of 'multiOutFlag'.
//  7/11/2002 -- [KF]  Added real version of 'multiOutFlag'.
//  7/13/2002 -- [KF]  Added support for "End Time"
//  7/16/2002 -- [KF]  Update for JEvalResp v0.974Beta:  Uses
//                     'processAndOutput()' with new parameter
//                     instead of 'preprocessParams()'.
//   8/6/2002 -- [ET]  Added support for "Headers:" check-box.
//  3/27/2003 -- [KF]  Added output directory.
//  4/11/2005 -- [ET]  Added 'useDelay' and 'showInput' options.
// 10/31/2005 -- [ET]  Added List-blockette interpolation options.
//  7/15/2009 -- [KF]  Modified to support running
//                     JPlotResp from another application.
//  5/26/2010 -- [ET]  Added "Unwrap Phase" and "Use Total Sensit" checkbox
//                     items.
//   4/2/2012 -- [ET]  Added support for "Multi-server plots" option.
// 10/29/2013 -- [ET]  Added 'b62_x' field.
//

package com.isti.jplotresp;

import java.io.File;
import java.io.PrintStream;

/**
 * Class RespProcessingThread creates a new thread to process parameters.
 */
public class RespProcessingThread implements Runnable
{
  private static int threadInitNumber = 0;
  /**
   * Returns (and increments) the next thread number for autonumbering
   * anonymous threads.
   * @return the next thread number.
   */
  protected static synchronized int nextThreadNum()
  {
    return threadInitNumber++;
  }

  private final PlotRun plotRunObj;          //PlotRun object
  private final RespEntryPanel ePanel;       //parent object
  private final boolean plotFlag;            //true for plot, false for file

  // parameters needed for the PlotRun methods
  private final String staListStr;
  private final String chaListStr;
  private final String netListStr;
  private final String siteListStr;
  private final String yearStr;
  private final String julianDayStr;
  private final String timeStr;
  private final String endYearStr;
  private final String endDayStr;
  private final String endTimeStr;
  private final String minFreqStr;
  private final String maxFreqStr;
  private final String numberFreqsStr;
  private final boolean stdioFlag;
  private final String outUnitsConvStr;
  private final boolean multiOutFlag;
  private final boolean verboseFlag;
  private final boolean headersFlag;
  private final boolean useDelayFlag;
  private final boolean showInputFlag;
  private final String b62XValueStr;
  private final boolean multiSvrFlag;
  private final boolean unwrapPhaseFlag;
  private final boolean totalSensitFlag;
  private final boolean listInterpOutFlag;
  private final boolean listInterpInFlag;
  private final String listInterpTensionStr;
  private final String startStageStr;
  private final String stopStageStr;
  private final String logSpacingStr;
  private final String apOutputStr;
  private final String fileNameString;
  private final String propsFileString;
  private final File outputDirectory;

  /**
   * Constructs a 'RespProcessingThread' object that is used
   * to run the processing under a different thread.
   * @param plotRunObj is a PlotRun object.
   * @param ePanel is the parent RespEntryPanel object.
   * @param plotFlag true for plot output, false for files output.
   */
  public RespProcessingThread(PlotRun plotRunObj, RespEntryPanel ePanel,
                              boolean plotFlag)
  {
    this.plotRunObj = plotRunObj;      //save the PlotRun object
    this.ePanel = ePanel;              //save the ePanel object
    this.plotFlag = plotFlag;          //save the plot flag

    // get the parameters needed for the PlotRun processAndOutput method
    staListStr = ePanel.stationTextField.getText();
    chaListStr = ePanel.channelTextField.getText();
    netListStr = ePanel.networkTextField.getText();
    siteListStr = ePanel.locationTextField.getText();
    yearStr = ePanel.beginTimePanel.yearTextField.getText();
    julianDayStr = ePanel.beginTimePanel.julianDayTextField.getText();
    timeStr = ePanel.beginTimePanel.timeTextField.getText();
    endYearStr = ePanel.endTimePanel.yearTextField.getText();
    endDayStr = ePanel.endTimePanel.julianDayTextField.getText();
    endTimeStr = ePanel.endTimePanel.timeTextField.getText();
    minFreqStr = ePanel.minFreqTextField.getText();
    maxFreqStr = ePanel.maxFreqTextField.getText();
    numberFreqsStr = ePanel.numFreqsTextField.getText();
    stdioFlag = false;
    outUnitsConvStr = (String)ePanel.unitsComboBox.getSelectedItem();
    verboseFlag = ePanel.verboseCheckBox.isSelected();
    headersFlag = ePanel.headersCheckBox.isSelected();
    useDelayFlag = ePanel.useDelayCheckBox.isSelected();
    showInputFlag = ePanel.showInputCheckBox.isSelected();
    b62XValueStr = ePanel.b62XValueTextField.getText();
    multiSvrFlag = ePanel.multiSvrCheckBox.isSelected();
    unwrapPhaseFlag = ePanel.unwrapPhaseCheckBox.isSelected();
    totalSensitFlag = ePanel.totalSensitCheckBox.isSelected();
    listInterpOutFlag = ePanel.interpOutputCheckBox.isSelected();
    listInterpInFlag = ePanel.interpInputCheckBox.isSelected();
    listInterpTensionStr = ePanel.tensionTextField.getText();
    startStageStr = ePanel.startStageTextField.getText();
    stopStageStr = ePanel.endStageTextField.getText();
    logSpacingStr =
        (String)(ePanel.frequencySpacingComboBox.getSelectedItem());
    apOutputStr = (String)(ePanel.responseTypeComboBox.getSelectedItem());
    fileNameString = ePanel.filenameRadioButton.isSelected() ?
                     ePanel.filenameTextField.getText() : "";
    propsFileString = ePanel.serverRadioButton.isSelected() ?
                      ePanel.serverTextField.getText() : "";
    multiOutFlag = ePanel.multiOutCheckBox.isSelected();
    final String outputDirectoryText =
        ePanel.outputDirectoryTextField.getText().trim();
    if (outputDirectoryText.length() > 0)  //if output directory specified
    {
      outputDirectory = new File(outputDirectoryText);  //save the output directory
    }
    else
    {
      outputDirectory = null;  //no output directory
    }

    // start the execution of the thread
    plotRunObj.execute(this, "RespProcessingThread-"+nextThreadNum());
  }

  /**
   * Process and output and possibly plot.
   * @return return flag from JEvalResp.
   */
  private boolean processAndOutput()
  {
      plotRunObj.clearErrorMessage();  //clear any previous error msgs
      boolean returnFlag = false;
      if (Thread.currentThread().isInterrupted())
      {
        return returnFlag;
      }
      try
      {
        if(returnFlag=plotRunObj.processAndOutput(staListStr,
                 chaListStr, netListStr, siteListStr, yearStr, julianDayStr,
                 timeStr, minFreqStr, maxFreqStr, numberFreqsStr, stdioFlag,
                     outUnitsConvStr, multiOutFlag, headersFlag, endYearStr,
                          endDayStr, endTimeStr, verboseFlag, startStageStr,
                   stopStageStr, logSpacingStr, apOutputStr, fileNameString,
                                propsFileString, outputDirectory, !plotFlag,
                             useDelayFlag, showInputFlag, listInterpOutFlag,
                                     listInterpInFlag, listInterpTensionStr,
                                           unwrapPhaseFlag, totalSensitFlag,
                                                b62XValueStr, multiSvrFlag))
        {     //process and (possible) output OK
          if(Thread.currentThread().isInterrupted())
          {
            return returnFlag;
          }
          if(plotFlag)
          {   //plot flag is set; show output on graph
            returnFlag = plotRunObj.processAndPlot(
                                             ePanel.plotDisplayOptionPanel);
          }
        }
      }
      catch(Exception ex)
      {
        final PrintStream out = plotRunObj.getConsoleOut();
        out.println("Error processing plot:");
        ex.printStackTrace(out);
      }
      return returnFlag;
  }

  /** Executing method for thread. */
  public void run()
  {
    synchronized(plotRunObj)
    {    //only allow one thread access to the 'PlotRun' object at a time
      boolean returnFlag = processAndOutput();
      ePanel.processAndOutputCompleted(returnFlag);   //indicate completed
    }
  }
}
